/**
 * @file ADC.c
 * @date 2019-09-16
 *
 * @cond
 *********************************************************************************************************************
 * ADC interface for XMC4200 Platform2Go Kit v1.0
 *
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 *********************************************************************************************************************
 *
 * Change History
 * --------------
 *
 * 2019-09-16:
 *     - Initial version
 *
 * @endcond 
 *
 */

#include "xmc_vadc.h" 
#include "Board_ADC.h"

#define ADC_RESOLUTION        12        /* Number of A/D converter bits       */

/**
  \fn          int32_t ADC_Initialize (void)
  \brief       Initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Initialize (void) {
		
	//Enable the VADC module
	XMC_SCU_RESET_DeassertPeripheralReset(XMC_SCU_PERIPHERAL_RESET_VADC);
  VADC->CLC       =   0;                     /* enable the VADC module clock  */
  VADC->GLOBCFG   = ((1UL << 15) |           /* enable DIVA write control     */
                     (6UL <<  0)  );         /* set DIVA to 6                 */
 	
  //Enable arbitration slot 02, priority set to 1     
	XMC_VADC_GROUP_BackgroundEnableArbitrationSlot(VADC_G0);	
	
	//Add a channel into the scan sequence Group 0 channel 0 (P14.0)
  XMC_VADC_GLOBAL_BackgroundAddChannelToSequence(VADC,0,0);
	
	VADC_G0->ARBCFG = ((3UL << 16) |          /* ANONS normal operation        */ 
                     (3UL <<  0)  );        /* ANONC normal operation        */
	VADC->BRSMR = ((1UL <<  0)); 				      /* ENGT = 01B                    */
	
  return 0;
}

/**
  \fn          int32_t ADC_Uninitialize (void)
  \brief       De-initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Uninitialize (void) {
  //Disable the VADC module
	XMC_SCU_RESET_AssertPeripheralReset(XMC_SCU_PERIPHERAL_RESET_VADC);
	return 0;
}

/**
  \fn          int32_t ADC_StartConversion (void)
  \brief       Start conversion
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_StartConversion (void) {	
  // Generates conversion request
  XMC_VADC_GLOBAL_BackgroundTriggerConversion(VADC);	
  return 0;
}

/**
  \fn          int32_t ADC_ConversionDone (void)
  \brief       Check if conversion finished
  \returns
   - \b  0: conversion finished
   - \b -1: conversion in progress
*/
int32_t ADC_ConversionDone (void) {
  XMC_VADC_GROUP_STATE_t status;
  status = XMC_VADC_GROUP_IsConverterBusy(VADC_G0);
  return ((status == XMC_VADC_GROUP_STATE_IDLE) ? 0 : -1);
}

/**
  \fn          int32_t ADC_GetValue (void)
  \brief       Get converted value
  \returns
   - <b> >=0</b>: converted value
   - \b -1: conversion in progress or failed
*/
int32_t ADC_GetValue (void) {
	//Return the result value of the result register
	if (ADC_ConversionDone() != 0) return -1;
  return (int32_t) XMC_VADC_GROUP_GetResult(VADC_G0,0);
}

/**
  \fn          uint32_t ADC_GetResolution (void)
  \brief       Get resolution of Analog-to-Digital Converter
  \returns     Resolution (in bits)
*/
uint32_t ADC_GetResolution (void) {
  return ADC_RESOLUTION;
}
