/*
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 *
 */

/*------------------------------------------------------------------------------
 * Name:    main.c
 * Purpose: CAN Remote Transmission Example
 *----------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <xmc_gpio.h>
#include "Driver_CAN.h"

/* CAN Driver Node selection: range [0-5] */
#define  TX_CAN_NODE            0       /* Number of the CAN Node that replies on the Remote Transmission Request (RTR) */
#define  RX_CAN_NODE            1       /* Number of the CAN Node that sends the Remote Transmission Request (RTR) */

/* On-board LED pin definition */
#define  LED                    P4_0

#define _CAN_Driver_(n)         Driver_CAN##n
#define  CAN_Driver_(n)         _CAN_Driver_(n)
extern   ARM_DRIVER_CAN         CAN_Driver_(TX_CAN_NODE);
extern   ARM_DRIVER_CAN         CAN_Driver_(RX_CAN_NODE);
#define  ptrCAN_TX_NODE         (&CAN_Driver_(TX_CAN_NODE))
#define  ptrCAN_RX_NODE         (&CAN_Driver_(RX_CAN_NODE))

uint32_t                        rx_obj_idx = 0xFFFFFFFFU;
uint8_t                         rx_data[8];
ARM_CAN_MSG_INFO                rx_msg_info;
uint32_t                        tx_obj_idx = 0xFFFFFFFFU;
uint8_t                         tx_data[8];
ARM_CAN_MSG_INFO                tx_msg_info;

/*------------------------------------------------------------------------------
 *  CAN Interface Error Handler 
 *----------------------------------------------------------------------------*/ 
static void Error_Handler (void){
	while(1);
}

/*------------------------------------------------------------------------------
 *  CAN Interface Signal Unit Event Callback
 *----------------------------------------------------------------------------*/
void CAN_SignalUnitEvent (uint32_t event) {

  switch (event) 
	{
    case ARM_CAN_EVENT_UNIT_ACTIVE:
      break;
    case ARM_CAN_EVENT_UNIT_WARNING:
      break;
    case ARM_CAN_EVENT_UNIT_PASSIVE:
      break;
    case ARM_CAN_EVENT_UNIT_BUS_OFF:
      break;
  }
}

/*------------------------------------------------------------------------------
 *  CAN Interface Signal Object Event Callback
 *----------------------------------------------------------------------------*/
void CAN_SignalObjectEvent (uint32_t obj_idx, uint32_t event) {

  if (event == ARM_CAN_EVENT_RECEIVE)                                      /* If receive event */
	{                                             
    if (obj_idx == rx_obj_idx)                                             /* If receive object event */
		{                                                    
      ptrCAN_RX_NODE->MessageRead(rx_obj_idx, &rx_msg_info, rx_data, 8);   /* Read received message */
			if (memcmp(tx_data, rx_data, 1U) == 0)
			{				
				XMC_GPIO_SetOutputHigh(LED);
			}
			else
			{
				Error_Handler();
			}
    }
  }
}

int main (void) 
{
  ARM_CAN_CAPABILITIES     can_cap;
  ARM_CAN_OBJ_CAPABILITIES can_obj_cap;
  int32_t                  status;
  uint32_t                 i, num_objects;

	XMC_GPIO_SetMode(LED, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
	
  can_cap = ptrCAN_TX_NODE->GetCapabilities ();                                     /* Get CAN driver capabilities (both TX and RX node share the same capabilities) */
  num_objects = can_cap.num_objects;                                                /* Number of receive/transmit objects */

  status = ptrCAN_TX_NODE->Initialize(CAN_SignalUnitEvent, CAN_SignalObjectEvent);  /* Initialize transmitting node */
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }
  status = ptrCAN_RX_NODE->Initialize(CAN_SignalUnitEvent, CAN_SignalObjectEvent);  /* Initialize receiving node */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }

  status = ptrCAN_TX_NODE->PowerControl(ARM_POWER_FULL);                            /* Power-up CAN interface (powering-up CAN interface is perfomed once, */																			
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }                                /* while all nodes share the same CAN interface) */

  status = ptrCAN_TX_NODE->SetMode(ARM_CAN_MODE_INITIALIZATION);                    /* Set transmitting node into initialization mode */
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }
  status = ptrCAN_RX_NODE->SetMode(ARM_CAN_MODE_INITIALIZATION);                    /* Set receiving node into initialization mode */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }

  status = ptrCAN_TX_NODE->SetBitrate(ARM_CAN_BITRATE_NOMINAL,                      /* Set nominal bitrate for transmitting node */
                                      500000U,                                      /* Set bitrate to 500 kbit/s */                                  
                                      ARM_CAN_BIT_PROP_SEG(6U)   |          				/* Set propagation segment to 6 time quanta */
                                      ARM_CAN_BIT_PHASE_SEG1(1U) |          				/* Set phase segment 1 to 1 time quantum (sample point at 80% of bit time) */
                                      ARM_CAN_BIT_PHASE_SEG2(2U) |          				/* Set phase segment 2 to 2 time quantum (total bit time is 10 time quanta long) */
                                      ARM_CAN_BIT_SJW(2U));                 				/* Resynchronization jump width is same as phase segment 2 */
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
  status = ptrCAN_RX_NODE->SetBitrate(ARM_CAN_BITRATE_NOMINAL,              				/* Set nominal bitrate for receiving node*/
                                      500000U,                              				/* Set bitrate to 500 kbit/s */
                                      ARM_CAN_BIT_PROP_SEG(6U)   |          				/* Set propagation segment to 6 time quanta */
                                      ARM_CAN_BIT_PHASE_SEG1(1U) |          				/* Set phase segment 1 to 1 time quantum (sample point at 80% of bit time) */
                                      ARM_CAN_BIT_PHASE_SEG2(2U) |          				/* Set phase segment 2 to 2 time quantum (total bit time is 10 time quanta long) */
                                      ARM_CAN_BIT_SJW(2U));                 				/* Resynchronization jump width is same as phase segment 2 */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
	status = ptrCAN_TX_NODE->SetMode (ARM_CAN_MODE_LOOPBACK_INTERNAL);                /* Set transmitting node into loop-back operation mode */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
	status = ptrCAN_RX_NODE->SetMode (ARM_CAN_MODE_LOOPBACK_INTERNAL);                /* Set receiving node into loop-back operation mode */
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }

  for (i = 0U; i < num_objects; i++) {                                              /* Find first available objects for receive and transmit */
    can_obj_cap = ptrCAN_TX_NODE->ObjectGetCapabilities(i);                         /* Get object capabilities (both TX and RX node share the same capabilities) */
    if      ((tx_obj_idx == 0xFFFFFFFFU) && (can_obj_cap.tx == 1U)) { tx_obj_idx = i; }
    else if ((rx_obj_idx == 0xFFFFFFFFU) && (can_obj_cap.rx == 1U)) { rx_obj_idx = i; break; }
  }
  if ((rx_obj_idx == 0xFFFFFFFFU) || (tx_obj_idx == 0xFFFFFFFFU)) { Error_Handler(); }
	
	status = ptrCAN_TX_NODE->ObjectConfigure(tx_obj_idx, ARM_CAN_OBJ_RX_RTR_TX_DATA); /* Configure and allocate transmit object to the transmitting node */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }

  memset(&tx_msg_info, 0U, sizeof(ARM_CAN_MSG_INFO));                               /* Clear trasmit message info structure */
  tx_msg_info.id = ARM_CAN_EXTENDED_ID(0x12345678U);                                /* Set extended ID for transmit message */
  tx_data[0] = '1';  tx_data[1] = '2';                                              /* Prepare data to transmit */
  tx_data[2] = '3';  tx_data[3] = '4';
  tx_data[4] = '5';  tx_data[5] = '6';
  tx_data[6] = '7';  tx_data[7] = '8';
	
	ptrCAN_TX_NODE->MessageSend(tx_obj_idx, &tx_msg_info, tx_data, 8U);               /* Prepare the M0 for transmission once the RTR is received */
	
	/* Set filter to receive message with extended ID 0x12345678 */
  status = ptrCAN_RX_NODE->ObjectSetFilter(rx_obj_idx, ARM_CAN_FILTER_ID_EXACT_ADD, ARM_CAN_EXTENDED_ID(0x12345678U), 0U);
	
	status = ptrCAN_RX_NODE->ObjectConfigure(rx_obj_idx, ARM_CAN_OBJ_TX_RTR_RX_DATA); /* Configure and allocate receive object to the receiving node */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }

	memset(&tx_msg_info, 0, sizeof(ARM_CAN_MSG_INFO));                                /* Clear trasmit message info structure */
  tx_msg_info.id  = ARM_CAN_EXTENDED_ID(0x12345678U);                               /* Set ID of message */
  tx_msg_info.rtr = 1;                                                              /* Set RTR flag of message to send RTR */
  tx_msg_info.dlc = 1;                                                              /* Set data length code of message to 1 to request 1 data byte */
  ptrCAN_RX_NODE->MessageSend(rx_obj_idx, &tx_msg_info, 0, 0);                      /* Send RTR */
	
  while (1) {
  }
}

