/*
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 *
 */

/*------------------------------------------------------------------------------
 * Name:    CAN_Receiver.c
 * Purpose: CAN Transmission Example
 *----------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <xmc_gpio.h>
#include "Driver_CAN.h"

/* Receiving CAN driver
   NOTE: selected CAN driver shouldn�t be changed while the CAN NODE1 is connected 
   to the CAN transceiver located on the XMC1400 Boot kit board               */
extern   ARM_DRIVER_CAN         Driver_CAN0;  
#define  ptrCAN_RX_NODE         (&Driver_CAN0)
#define  LED                    P4_0             /* On-board LED pin definition */

uint32_t                        rx_obj_idx = 0xFFFFFFFFU;
uint8_t                         rx_data[8];
ARM_CAN_MSG_INFO                rx_msg_info;
uint8_t                         tx_data[8];      /* used to store expected data */

/* GPIO LED pin configuration */
XMC_GPIO_CONFIG_t LED_pin_config =
{
  .mode = XMC_GPIO_MODE_OUTPUT_PUSH_PULL,
  .output_level = XMC_GPIO_OUTPUT_LEVEL_HIGH
};

/*------------------------------------------------------------------------------
 *  CAN Interface Error Handler 
 *----------------------------------------------------------------------------*/ 
static void Error_Handler (void){
	while(1);
}

/*------------------------------------------------------------------------------
 *  CAN Interface Signal Unit Event Callback
 *----------------------------------------------------------------------------*/
void CAN_SignalUnitEvent (uint32_t event) {

  switch (event) 
	{
    case ARM_CAN_EVENT_UNIT_ACTIVE:
      break;
    case ARM_CAN_EVENT_UNIT_WARNING:
      break;
    case ARM_CAN_EVENT_UNIT_PASSIVE:
      break;
    case ARM_CAN_EVENT_UNIT_BUS_OFF:
      break;
  }
}

/*------------------------------------------------------------------------------
 *  CAN Interface Signal Object Event Callback
 *----------------------------------------------------------------------------*/
void CAN_SignalObjectEvent (uint32_t obj_idx, uint32_t event) {

  if (event == ARM_CAN_EVENT_RECEIVE)                     /* If receive event */
	{                                             
    if (obj_idx == rx_obj_idx)                            /* If receive object event */
		{                                                    
      ptrCAN_RX_NODE->MessageRead(rx_obj_idx, &rx_msg_info, rx_data, 1U);
			if (memcmp(tx_data, rx_data, 1U) == 0)
			{				
				XMC_GPIO_ToggleOutput(LED);
			}
			else
			{
				Error_Handler();
			}
    }
  }
}

int main (void) 
{
  ARM_CAN_CAPABILITIES     can_cap;
  ARM_CAN_OBJ_CAPABILITIES can_obj_cap;
  int32_t                  status;
  uint32_t                 i, num_objects;

	XMC_GPIO_Init(LED, &LED_pin_config);
	
	can_cap = ptrCAN_RX_NODE->GetCapabilities ();                                     /* Get CAN driver capabilities */
  num_objects = can_cap.num_objects;                                                /* Number of receive/transmit objects */
	
	status = ptrCAN_RX_NODE->Initialize(CAN_SignalUnitEvent, CAN_SignalObjectEvent);  /* Initialize receiving node */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
	status = ptrCAN_RX_NODE->PowerControl(ARM_POWER_FULL);                            /* Power-up CAN interface */																			
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }                                
	
	status = ptrCAN_RX_NODE->SetMode(ARM_CAN_MODE_INITIALIZATION);                    /* Set receiving node into initialization mode */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
	status = ptrCAN_RX_NODE->SetBitrate(ARM_CAN_BITRATE_NOMINAL,              				/* Set nominal bitrate for receiving node*/
                                      500000U,                              				/* Set bitrate to 500 kbit/s */
                                      ARM_CAN_BIT_PROP_SEG(6U)   |          				/* Set propagation segment to 6 time quanta */
                                      ARM_CAN_BIT_PHASE_SEG1(1U) |          				/* Set phase segment 1 to 1 time quantum (sample point at 80% of bit time) */
                                      ARM_CAN_BIT_PHASE_SEG2(2U) |          				/* Set phase segment 2 to 2 time quantum (total bit time is 10 time quanta long) */
                                      ARM_CAN_BIT_SJW(2U));                 				/* Resynchronization jump width is same as phase segment 2 */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
	status = ptrCAN_RX_NODE->SetMode (ARM_CAN_MODE_NORMAL);                						/* Set receiving node into normal operation mode */
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
	for (i = 0U; i < num_objects; i++) {                                              /* Find first available object for receive */
    can_obj_cap = ptrCAN_RX_NODE->ObjectGetCapabilities(i);                         /* Get object capabilities */
    if ((rx_obj_idx == 0xFFFFFFFFU) && (can_obj_cap.rx == 1U)) { rx_obj_idx = i; break; }
  }
  if (rx_obj_idx == 0xFFFFFFFFU) { Error_Handler(); }	

  /* Set filter to receive messages with extended ID 0x12345678 to receive object;
	   extended ID should be the same as one used in "CAN_Transmitter.c"      */
  status = ptrCAN_RX_NODE->ObjectSetFilter(rx_obj_idx, ARM_CAN_FILTER_ID_EXACT_ADD, ARM_CAN_EXTENDED_ID(0x12345678U), 0U);
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
	status = ptrCAN_RX_NODE->ObjectConfigure(rx_obj_idx, ARM_CAN_OBJ_RX);             /* Configure and allocate receive object to the receiving node */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }	
	
	tx_data[0] = 0xFFU;                                                               /* Prepare expected data; should equal the transmitted data from "CAN_Transmitter.c"*/
	
	while (1) 
	{
  }
}
