/*
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 *
 */

/*------------------------------------------------------------------------------
 * Name:    CAN_Transmitter.c
 * Purpose: CAN Transmission Example
 *----------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <xmc_gpio.h>
#include "Driver_CAN.h"

/* Transmitting CAN driver
   NOTE: selected CAN driver shouldn�t be changed while the CAN NODE1 is connected 
   to the CAN transceiver located on the XMC1400 Boot kit board               */
extern   ARM_DRIVER_CAN         Driver_CAN0;  
#define  ptrCAN_TX_NODE         (&Driver_CAN0)
#define  LED                    P4_0          /* On-board LED pin definition  */

uint32_t                        tx_obj_idx = 0xFFFFFFFFU;
uint8_t                         tx_data[8];
ARM_CAN_MSG_INFO                tx_msg_info;

/* GPIO LED pin configuration */
XMC_GPIO_CONFIG_t LED_pin_config =
{
  .mode = XMC_GPIO_MODE_OUTPUT_PUSH_PULL,
  .output_level = XMC_GPIO_OUTPUT_LEVEL_HIGH
};

/*------------------------------------------------------------------------------
 *  CAN Interface Error Handler 
 *----------------------------------------------------------------------------*/ 
static void Error_Handler (void){
	while(1)
	{
		/* code execution should not reach this point */
	}
}

/*------------------------------------------------------------------------------
 *  CAN Interface Signal Unit Event Callback
 *----------------------------------------------------------------------------*/
void CAN_SignalUnitEvent (uint32_t event) {
	ARM_CAN_STATUS status;

  switch (event) 
	{
    case ARM_CAN_EVENT_UNIT_ACTIVE:
      break;
    case ARM_CAN_EVENT_UNIT_WARNING:
			
			status = ptrCAN_TX_NODE->GetStatus();
		  if (status.last_error_code == ARM_CAN_LEC_ACK_ERROR){
			  /* make sure that CAN transmitter is properly connected to the receiving board */
			  Error_Handler();
			}
    case ARM_CAN_EVENT_UNIT_PASSIVE:
      break;
    case ARM_CAN_EVENT_UNIT_BUS_OFF:
      break;
  }
}

/*------------------------------------------------------------------------------
 *  CAN Interface Signal Object Event Callback
 *----------------------------------------------------------------------------*/
void CAN_SignalObjectEvent (uint32_t obj_idx, uint32_t event) {

  if (event == ARM_CAN_EVENT_SEND_COMPLETE)    /* If the message was sent successfully */
	{                                             
	  XMC_GPIO_ToggleOutput(LED);
	}
	else
	{
		Error_Handler();
	}
}

int main (void) 
{
  ARM_CAN_CAPABILITIES     can_cap;
  ARM_CAN_OBJ_CAPABILITIES can_obj_cap;
  int32_t                  status;
  uint32_t                 i, num_objects;

	XMC_GPIO_Init(LED, &LED_pin_config);
	
  can_cap = ptrCAN_TX_NODE->GetCapabilities ();    																	/* Get CAN driver capabilities */
  num_objects = can_cap.num_objects;                                                /* Number of receive/transmit objects */

  status = ptrCAN_TX_NODE->Initialize(CAN_SignalUnitEvent, CAN_SignalObjectEvent);  /* Initialize transmitting node */
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }

  status = ptrCAN_TX_NODE->PowerControl(ARM_POWER_FULL);                            /* Power-up CAN interface */																			
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }                                

  status = ptrCAN_TX_NODE->SetMode(ARM_CAN_MODE_INITIALIZATION);                    /* Set transmitting node into initialization mode */
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }
 
  status = ptrCAN_TX_NODE->SetBitrate(ARM_CAN_BITRATE_NOMINAL,                      /* Set nominal bitrate for transmitting node */
                                      500000U,                                      /* Set bitrate to 500 kbit/s */                                  
                                      ARM_CAN_BIT_PROP_SEG(6U)   |          				/* Set propagation segment to 6 time quanta */
                                      ARM_CAN_BIT_PHASE_SEG1(1U) |          				/* Set phase segment 1 to 1 time quantum (sample point at 80% of bit time) */
                                      ARM_CAN_BIT_PHASE_SEG2(2U) |          				/* Set phase segment 2 to 2 time quantum (total bit time is 10 time quanta long) */
                                      ARM_CAN_BIT_SJW(2U));                 				/* Resynchronization jump width is same as phase segment 2 */
	if (status != ARM_DRIVER_OK ) { Error_Handler(); }

	status = ptrCAN_TX_NODE->SetMode (ARM_CAN_MODE_NORMAL);                           /* Set transmitting node into normal operation mode */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }	
	
  for (i = 0U; i < num_objects; i++) {                                              /* Find first available object for transmit */
    can_obj_cap = ptrCAN_TX_NODE->ObjectGetCapabilities(i);                         /* Get object capabilities */
    if ((tx_obj_idx == 0xFFFFFFFFU) && (can_obj_cap.tx == 1U)) { tx_obj_idx = i; break; }
  }
  if (tx_obj_idx == 0xFFFFFFFFU) { Error_Handler(); }	

	status = ptrCAN_TX_NODE->ObjectConfigure(tx_obj_idx, ARM_CAN_OBJ_TX);             /* Configure and allocate transmit object to the transmitting node */
  if (status != ARM_DRIVER_OK ) { Error_Handler(); }
	
	memset(&tx_msg_info, 0U, sizeof(ARM_CAN_MSG_INFO));                               /* Clear message info structure */
  tx_msg_info.id = ARM_CAN_EXTENDED_ID(0x12345678U);                                /* Set extended ID for transmit message */
  tx_data[0]     = 0xFFU;                                                           /* Initialize transmit data */
	
	ptrCAN_TX_NODE->MessageSend(tx_obj_idx, &tx_msg_info, tx_data, 1U);               /* Send data message with 1 data byte */
	
  while (1) 
	{
  }
}	
