/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network:Service
 * Copyright (c) 2004-2015 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    Telnet_Server_UIF.c
 * Purpose: Telnet Server User Interface
 * Rev.:    V7.0.0
 *----------------------------------------------------------------------------*/
//! [code_Telnet_Server_UIF]
#include <stdio.h>

#include "RTE_Components.h"
#include "BOARD_Led.h"

#include "rl_net.h"
 
// \brief Request a message for a Telnet server session.
// \param[in]     msg           code of requested message.
// \param[out]    buf           output buffer to write the message to.
// \param[in]     buf_len       length of the output buffer in bytes.
// \return        number of bytes written to output buffer.
uint32_t netTELNETs_ProcessMessage (netTELNETs_Message msg, char *buf, uint32_t buf_len) {
  uint32_t len = 0;
 
  switch (msg) {
    case netTELNETs_MessageWelcome:
      // Initial welcome message
      len = sprintf (buf, "\r\n"
                          "Embedded Telnet Server\r\n");
      break;
 
    case netTELNETs_MessagePrompt:
      // Prompt message
      len = sprintf (buf, "\r\n"
                          "Cmd> ");
      break;
 
    case netTELNETs_MessageLogin:
      // Login message, if authentication is enabled
      len = sprintf (buf, "\r\n"
                          "Please login...");
      break;
 
    case netTELNETs_MessageUsername:
      // Username request login message
      len = sprintf (buf, "\r\n"
                          "Username: ");
      break;
 
    case netTELNETs_MessagePassword:
      // Password request login message
      len = sprintf (buf, "\r\n"
                          "Password: ");
      break;
 
    case netTELNETs_MessageLoginFailed:
      // Incorrect login error message
      len = sprintf (buf, "\r\n"
                          "Login incorrect");
      break;
 
    case netTELNETs_MessageLoginTimeout:
      // Login timeout error message
      len = sprintf (buf, "\r\n"
                          "Login timeout\r\n");
      break;
 
    case netTELNETs_MessageUnsolicited:
      // Unsolicited message (ie. from basic interpreter)
      len = sprintf (buf, "\r\n"
                          "Unsolicited message\r\n");
      break;
  }
  return (len);
}
 
// \brief Process and execute a command requested by the Telnet client.
// \param[in]     cmd           pointer to command string from Telnet client.
// \param[out]    buf           output buffer to write the return message to.
// \param[in]     buf_len       length of the output buffer in bytes.
// \param[in,out] pvar          pointer to a session's local buffer of 4 bytes.
//                              - 1st call = cleared to 0,
//                              - 2nd call = not altered by the system,
//                              - 3rd call = not altered by the system, etc.
// \return        number of bytes written to output buffer.
//                - return len | (1u<<31) = repeat flag, the system calls this function
//                                          again for the same command.
//                - return len | (1u<<30) = disconnect flag, the system disconnects
//                                          the Telnet client.
uint32_t netTELNETs_ProcessCommand (const char *cmd, char *buf, uint32_t buf_len, uint32_t *pvar) {
  uint32_t len = 0;
 
  if (netTELNETs_CheckCommand(cmd, "LED1=ON") == true) 
	{
    // Switch ON LED1
		LED_On(0);
    len = sprintf (buf," Ok");
    return (len);
  }
  else if (netTELNETs_CheckCommand(cmd, "LED1=OFF") == true) 
	{
    // Switch OFF LED1
		LED_Off(0);
    len = sprintf (buf," Ok");
    return (len);
  }
  else if (netTELNETs_CheckCommand(cmd, "LED2=ON") == true) 
	{
    // Switch ON LED2
		LED_On(1);
    len = sprintf (buf," Ok");
    return (len);
  }
  else if (netTELNETs_CheckCommand(cmd, "LED2=OFF") == true) 
	{
    // Switch OFF LED2
		LED_Off(1);
    len = sprintf (buf," Ok");
    return (len);
  }
  else if ((netTELNETs_CheckCommand(cmd, "HELP") == true) || (netTELNETs_CheckCommand(cmd, "?") == true)) 
	{
    // Generate reply and disconnect
    len = sprintf (buf, "\r\n"
		                    "    Available commands:\r\n"
		                    "    --------------------------------------\r\n" 
		                    "    led1=<on|off> - switch on/off LED1\r\n" 
		                    "    led2=<on|off> - switch on/off LED2\r\n" 
		                    "    help , ?      - disaplay this help\r\n" 		
		                    "    bye           - disconnect\r\n" 		
		                    "\r\n"
                        "    <ESC>,<^C>    - disconnect\r\n" 				
                        "    <BS>          - delete character left\r\n" 				
                        "    <UP><DOWN>    - recall command history\r\n" 				
		                    "\r\n");
    return (len);
  }
  else if (netTELNETs_CheckCommand(cmd, "BYE") == true) 
	{
    // Generate reply and disconnect
    len = sprintf (buf, "\r\n"
                        "Disconnecting\r\n");
    return (len | (1u<<30));
  }
  else 
	{
    // Unknown command given
    len = sprintf (buf, "\r\n==> Unknown Command: %s", cmd);
	}
	
  return (len);
}
//! [code_Telnet_Server_UIF]
