/*----------------------------------------------------------------------------
 * Name:    LED_XMC4500.c
 * Purpose: LED interface for Infineon XMC4500 Relax Kit
 * Rev.:    1.00
 *----------------------------------------------------------------------------*/
 
 /* Copyright (c) 2013 - 2015 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "xmc_gpio.h"                        
#include "Board_LED.h" 

#define LED_COUNT 2
#define LED_1_PORT_PIN			P1_1
#define LED_2_PORT_PIN			P1_0

// LED initialization
XMC_GPIO_CONFIG_t led_config;			 

/**
  \fn          int32_t LED_Initialize (void)
  \brief       Initialize LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Initialize (void) {
  /* configure PORT P1.0..1 pin as output push pull */
	led_config.mode = XMC_GPIO_MODE_OUTPUT_PUSH_PULL;
	led_config.output_level = XMC_GPIO_OUTPUT_LEVEL_LOW;
	led_config.output_strength = XMC_GPIO_OUTPUT_STRENGTH_STRONG_SOFT_EDGE;
	XMC_GPIO_Init(LED_1_PORT_PIN, &led_config);
	XMC_GPIO_Init(LED_2_PORT_PIN, &led_config);
	LED_SetOut (0);  
	return 0;
}

/**
  \fn          int32_t LED_Uninitialize (void)
  \brief       De-initialize LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Uninitialize (void) {
	/* configure PORT P1.0..1 pin with default values  */
	led_config.mode = (XMC_GPIO_MODE_t) 0;
	led_config.output_level = (XMC_GPIO_OUTPUT_LEVEL_t) 0;
	led_config.output_strength = (XMC_GPIO_OUTPUT_STRENGTH_t) 0;
	XMC_GPIO_Init(LED_1_PORT_PIN, &led_config);
	XMC_GPIO_Init(LED_2_PORT_PIN, &led_config);
  return 0;
}

/**
  \fn          int32_t LED_On (uint32_t num)
  \brief       Turn on requested LED
  \param[in]   num  LED number
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_On (uint32_t num) {
	switch (num) {
		case 0:
			XMC_GPIO_SetOutputLevel(LED_1_PORT_PIN, XMC_GPIO_OUTPUT_LEVEL_HIGH);
			return 0;
		case 1:
			XMC_GPIO_SetOutputLevel(LED_2_PORT_PIN, XMC_GPIO_OUTPUT_LEVEL_HIGH);
			return 0;
		default:
			return -1;	
	}
}

/**
  \fn          int32_t LED_Off (uint32_t num)
  \brief       Turn off requested LED
  \param[in]   num  LED number
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Off (uint32_t num) {
	switch (num) {
		case 0:
			XMC_GPIO_SetOutputLevel(LED_1_PORT_PIN, XMC_GPIO_OUTPUT_LEVEL_LOW);
			return 0;
		case 1:
			XMC_GPIO_SetOutputLevel(LED_2_PORT_PIN, XMC_GPIO_OUTPUT_LEVEL_LOW);
			return 0;
		default:
			return -1;	
	}
}

/**
  \fn          int32_t LED_SetOut (uint32_t val)
  \brief       Write value to LEDs
  \param[in]   val  value to be displayed on LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_SetOut (uint32_t val) {
  uint8_t n;
  for (n = 0; n < LED_COUNT; n++) {
    if (val & (1 << n)) LED_On (n);
    else                LED_Off(n);
  }
	return 0;
}

/**
  \fn          uint32_t LED_GetCount (void)
  \brief       Get number of LEDs
  \return      Number of available LEDs
*/
uint32_t LED_GetCount (void) {
  return LED_COUNT;
}
