/*
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 *
 */

/********************************************************************************
**                                                                            **
**                                                                            **
** PLATFORM : Infineon XMC1200 Series                                         **
**                                                                            **
** AUTHOR : Attila Tomasovics				                                  **
**                                                                            **
** version 2 (RGB LED Shield, standalone, XMCLib)				              **
** MODIFICATION DATE : 29 July 2015	                                          **
**                                                                            **
*******************************************************************************/


#include "xmc_acmp.h"
#include "xmc_gpio.h"
#include "xmc_bccu.h"
#include "xmc_ccu4.h"
#include "xmc_scu.h"

// CURRENT CONTROL PARAMATERS
// off-times
#define OFFTIME_RED		0x38
#define OFFTIME_GREEN	0x39
#define OFFTIME_BLUE	0x38
// peak-current references
#define CURRENT_RED		0x15
#define CURRENT_GREEN	0x15
#define CURRENT_BLUE	0x15

// linear walk time: 492ms
#define WALKTIME		48
// 0-100% dimming time: 491ms
#define FADERATE		7

// PDM outputs to be low pass filtered for peak-current references
#define R_DAC P0_4
#define G_DAC P0_12
#define B_DAC P0_6

// comparator output pins
#define R_CMP_OUT P0_10
#define G_CMP_OUT P1_0
#define B_CMP_OUT P0_5

// CCU4 clear input pins
#define R_CLR P0_9
#define G_CLR P0_8
#define B_CLR P0_7

// CCU4 output pins (connected to MOSFET gates)
#define R_GATE P1_3
#define G_GATE P1_2
#define B_GATE P1_1


/* Clock configuration */
XMC_SCU_CLOCK_CONFIG_t clock_config =
{
		.pclk_src = XMC_SCU_CLOCK_PCLKSRC_DOUBLE_MCLK,
		.fdiv = 0,
		.idiv = 1
};

/* ACMP configuration */
XMC_ACMP_CONFIG_t cmp_config =
{
	.filter_disable = 0U,
    .output_invert  = 0U,
	.hysteresis     = XMC_ACMP_HYSTERESIS_10
};

/* BCCU configuration */
XMC_BCCU_GLOBAL_CONFIG_t bccu_global_config =
{
  .fclk_ps = 0x50, //800kHz @PCLK=64MHz
  .dclk_ps = 0xdb, //292.237KHz @PCLK=64MHz
  .bclk_sel = XMC_BCCU_BCLK_MODE_NORMAL, //200KHz @PCLK=64MHz
  .maxzero_at_output = 200
};
XMC_BCCU_CH_CONFIG_t dac_config =
{
  .pack_en     	   = 0,
  .dim_sel         = 0x7,	// globdim
  .dim_bypass	   = 1,		// bypass
  .flick_wd_en     = 0x0
};
XMC_BCCU_CH_CONFIG_t r_pdm_config =
{
  .pack_en     	   = 1,
  .dim_sel         = 0x0,	// DE0
  .dim_bypass	   = 0,		// no bypass
  .flick_wd_en     = 1,
  .pack_offcnt_val = 0,
  .pack_offcmp_lev = 199,
  .pack_oncmp_lev  = 3
};
XMC_BCCU_CH_CONFIG_t g_pdm_config =
{
  .pack_en     	   = 1,
  .dim_sel         = 0x0,	// DE0
  .dim_bypass	   = 0,		// no bypass
  .flick_wd_en     = 1,
  .pack_offcnt_val = 66,
  .pack_offcmp_lev = 199,
  .pack_oncmp_lev  = 3
};
XMC_BCCU_CH_CONFIG_t b_pdm_config =
{
  .pack_en     	   = 1,
  .dim_sel         = 0x0,	// DE0
  .dim_bypass	   = 0,		// no bypass
  .flick_wd_en     = 1,
  .pack_offcnt_val = 132,
  .pack_offcmp_lev = 199,
  .pack_oncmp_lev  = 3
};
XMC_BCCU_DIM_CONFIG_t bccu_dim_config =
{
  .dim_div = 0x10
};

/* CCU4 configuration */
XMC_CCU4_SLICE_COMPARE_CONFIG_t timer_config =
{
  .timer_mode 		   = XMC_CCU4_SLICE_TIMER_COUNT_MODE_EA,
  .monoshot   		   = 0,		/* no single shot */
  .shadow_xfer_clear   = 0U,
  .dither_timer_period = 0U,
  .dither_duty_cycle   = 0U,
  .prescaler_mode	   = XMC_CCU4_SLICE_PRESCALER_MODE_NORMAL,
  .mcm_enable		   = 1U,	/* multi-channel mode for PDM dimming*/
  .prescaler_initval   = 0U,
  .float_limit		   = 0U,
  .dither_limit		   = 0U,
  .passive_level 	   = XMC_CCU4_SLICE_OUTPUT_PASSIVE_LEVEL_LOW,
  .timer_concatenation = 0U
};
XMC_CCU4_SLICE_EVENT_CONFIG_t slice_event_config =
{
	.mapped_input	= XMC_CCU4_SLICE_INPUT_B,
	.edge			= XMC_CCU4_SLICE_EVENT_EDGE_SENSITIVITY_RISING_EDGE,
	.duration		= XMC_CCU4_SLICE_EVENT_FILTER_DISABLED
};


int main(void)
{

	//----CLOCK-SETUP-----------------------------------------------------------------------------------
	XMC_SCU_CLOCK_Init(&clock_config);
	//--------------------------------------------------------------------------------------------------

	/* BCCU Configuration */
	XMC_BCCU_GlobalInit(BCCU0, &bccu_global_config);

	//---DACs---------------------------------------------------------------------------------------------------
	/* init DACs */
	XMC_BCCU_CH_Init(BCCU0_CH0, &dac_config);
	XMC_BCCU_CH_Init(BCCU0_CH6, &dac_config);
	XMC_BCCU_CH_Init(BCCU0_CH2, &dac_config);

	/* Set immediate linear walk for DACs*/
	XMC_BCCU_CH_SetLinearWalkPrescaler(BCCU0_CH0, 0);
	XMC_BCCU_CH_SetLinearWalkPrescaler(BCCU0_CH6, 0);
	XMC_BCCU_CH_SetLinearWalkPrescaler(BCCU0_CH2, 0);

	/* Enable DAC channels*/
	XMC_BCCU_ConcurrentEnableChannels(BCCU0, 0x45);

	// set current references to low now
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH0, CURRENT_RED);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH6, CURRENT_GREEN);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH2, CURRENT_BLUE);
	XMC_BCCU_ConcurrentStartLinearWalk(BCCU0, 0x45);

	XMC_GPIO_SetMode(R_DAC, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT1);
	XMC_GPIO_SetMode(G_DAC, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT1);
	XMC_GPIO_SetMode(B_DAC, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT1);
	//----------------------------------------------------------------------------------------------------------


	//----COMPARATORs-------------------------------------------------------------------------------------------
	/* Enable and Initialize comparators */
	XMC_ACMP_SetInput(XMC_ACMP0, 0, XMC_ACMP_INP_SOURCE_STANDARD_PORT);
	XMC_ACMP_Init(XMC_ACMP0, 0, &cmp_config);
	XMC_ACMP_EnableComparator(XMC_ACMP0, 0);

	XMC_ACMP_SetInput(XMC_ACMP0, 1, XMC_ACMP_INP_SOURCE_STANDARD_PORT);
	XMC_ACMP_Init(XMC_ACMP0, 1, &cmp_config);
	XMC_ACMP_EnableComparator(XMC_ACMP0, 1);

	XMC_ACMP_SetInput(XMC_ACMP0, 2, XMC_ACMP_INP_SOURCE_STANDARD_PORT);
	XMC_ACMP_Init(XMC_ACMP0, 2, &cmp_config);
	XMC_ACMP_EnableComparator(XMC_ACMP0, 2);

	XMC_GPIO_SetMode(R_CMP_OUT, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT4);
	XMC_GPIO_SetMode(G_CMP_OUT, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT6);
	XMC_GPIO_SetMode(B_CMP_OUT, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT6);
	//----------------------------------------------------------------------------------------------------------


	//----MOSFET-SWITCHING--------------------------------------------------------------------------------------
	/* Ensure that fCCU reaches CCU40 */
	XMC_CCU4_SetModuleClock(CCU40, XMC_CCU4_CLOCK_SCU);

	XMC_CCU4_Init(CCU40, XMC_CCU4_SLICE_MCMS_ACTION_TRANSFER_PR_CR);

	/* Get the slices out of idle mode */
	XMC_CCU4_EnableClock(CCU40, 3);
	XMC_CCU4_EnableClock(CCU40, 2);
	XMC_CCU4_EnableClock(CCU40, 1);

	/* Start the prescaler and restore clocks to slices */
	XMC_CCU4_StartPrescaler(CCU40);

	/* Initialize the slices */
	XMC_CCU4_SLICE_CompareInit(CCU40_CC43, &timer_config);
	XMC_CCU4_SLICE_CompareInit(CCU40_CC42, &timer_config);
	XMC_CCU4_SLICE_CompareInit(CCU40_CC41, &timer_config);

	/* Program a period and compare values */
	XMC_CCU4_SLICE_SetTimerPeriodMatch(CCU40_CC43, 0x27FU);
	XMC_CCU4_SLICE_SetTimerCompareMatch(CCU40_CC43, OFFTIME_RED);
	XMC_CCU4_SLICE_SetTimerPeriodMatch(CCU40_CC42, 0x27FU);
	XMC_CCU4_SLICE_SetTimerCompareMatch(CCU40_CC42, OFFTIME_GREEN);
	XMC_CCU4_SLICE_SetTimerPeriodMatch(CCU40_CC41, 0x27FU);
	XMC_CCU4_SLICE_SetTimerCompareMatch(CCU40_CC41, OFFTIME_BLUE);

	/* Enable shadow transfer */
	XMC_CCU4_EnableShadowTransfer(CCU40, XMC_CCU4_SHADOW_TRANSFER_SLICE_3);
	XMC_CCU4_EnableShadowTransfer(CCU40, XMC_CCU4_SHADOW_TRANSFER_SLICE_2);
	XMC_CCU4_EnableShadowTransfer(CCU40, XMC_CCU4_SHADOW_TRANSFER_SLICE_1);

	// external stop configuration (peak-current control)
	XMC_CCU4_SLICE_ConfigureEvent(CCU40_CC43, XMC_CCU4_SLICE_EVENT_0, &slice_event_config);
	XMC_CCU4_SLICE_StopConfig(CCU40_CC43, XMC_CCU4_SLICE_EVENT_0, XMC_CCU4_SLICE_END_MODE_TIMER_CLEAR);
	XMC_CCU4_SLICE_ConfigureEvent(CCU40_CC42, XMC_CCU4_SLICE_EVENT_0, &slice_event_config);
	XMC_CCU4_SLICE_StopConfig(CCU40_CC42, XMC_CCU4_SLICE_EVENT_0, XMC_CCU4_SLICE_END_MODE_TIMER_CLEAR);
	XMC_CCU4_SLICE_ConfigureEvent(CCU40_CC41, XMC_CCU4_SLICE_EVENT_0, &slice_event_config);
	XMC_CCU4_SLICE_StopConfig(CCU40_CC41, XMC_CCU4_SLICE_EVENT_0, XMC_CCU4_SLICE_END_MODE_TIMER_CLEAR);

	// start switching
	XMC_CCU4_SLICE_StartTimer(CCU40_CC43);
	XMC_CCU4_SLICE_StartTimer(CCU40_CC42);
	XMC_CCU4_SLICE_StartTimer(CCU40_CC41);

	// enable pins
	XMC_GPIO_SetMode(R_GATE, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT2);
	XMC_GPIO_SetMode(G_GATE, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT2);
	XMC_GPIO_SetMode(B_GATE, XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT2);
	//----------------------------------------------------------------------------------------------------------


	//----DIMMING&COLOR-----------------------------------------------------------------------------------------
	// init dimming engine
	XMC_BCCU_DIM_Init(BCCU0_DE0, &bccu_dim_config);
	// enable dimming engine
	XMC_BCCU_EnableDimmingEngine(BCCU0, XMC_BCCU_CH_DIMMING_SOURCE_DE0);

	/* init PDMs */
	XMC_BCCU_CH_Init(BCCU0_CH5, &r_pdm_config);
	XMC_BCCU_CH_Init(BCCU0_CH4, &g_pdm_config);
	XMC_BCCU_CH_Init(BCCU0_CH3, &b_pdm_config);
	/* Set fast linear walk for PDMs*/
	XMC_BCCU_CH_SetLinearWalkPrescaler(BCCU0_CH5, WALKTIME);
	XMC_BCCU_CH_SetLinearWalkPrescaler(BCCU0_CH4, WALKTIME);
	XMC_BCCU_CH_SetLinearWalkPrescaler(BCCU0_CH3, WALKTIME);

	/* Enable PDM channels*/
	XMC_BCCU_ConcurrentEnableChannels(BCCU0, 0x38);

	// set initial color to white
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH5, 0x555);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH4, 0x555);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH3, 0x555);
	XMC_BCCU_ConcurrentStartLinearWalk(BCCU0, 0x38);

	// go to maximum dimming level
	XMC_BCCU_DIM_SetTargetDimmingLevel(BCCU0_DE0, 4095);
	XMC_BCCU_StartDimming(BCCU0, 0);
	//----------------------------------------------------------------------------------------------------------

	/* 500ms timer interrupt */
	SysTick_Config(SystemCoreClock>>1);


	while(1)
	{

	}
}


/* 500ms interrupt */
void SysTick_Handler(void)
{
  static uint8_t step = 0;

  /* Change to yellow */
  if (++step == 1)
  {
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH5, 0x800);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH4, 0x800);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH3, 0x000);
	XMC_BCCU_ConcurrentStartLinearWalk(BCCU0, 0x38);
  }
  /* Change to magenta */
  else if (step == 2)
  {
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH5, 0x800);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH4, 0x000);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH3, 0x800);
	XMC_BCCU_ConcurrentStartLinearWalk(BCCU0, 0x38);
  }
  /* Change to cyan */
  else if (step == 3)
  {
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH5, 0x000);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH4, 0x800);
	XMC_BCCU_CH_SetTargetIntensity(BCCU0_CH3, 0x800);
	XMC_BCCU_ConcurrentStartLinearWalk(BCCU0, 0x38);
	step = 0;
  }
}
