/**
 * @file main.c
 * @date 2019-04-29
 * @brief XMC1400 Bootkit SCU wakeup example
 *
 * @cond
 *****************************************************************************
 * XMClib v2.2.0 - XMC Peripheral Driver Library
 *
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 *****************************************************************************
 *
 * Change History
 * --------------
 *
 * 2019-04-29:
 *     - Initial version
 *
 * @endcond
 *
 */

#include "xmc_gpio.h"
#include "xmc_eru.h"

#define LED P4_0
#define POT P2_5

static const XMC_ERU_ETL_CONFIG_t button_event_generator_config =
{
  .input = ERU0_ETL1_INPUTA_P2_5,
  .source = XMC_ERU_ETL_SOURCE_A,
  .edge_detection = XMC_ERU_ETL_EDGE_DETECTION_FALLING,
  .status_flag_mode = XMC_ERU_ETL_STATUS_FLAG_MODE_HWCTRL,
  .enable_output_trigger = true,
  .output_trigger_channel = XMC_ERU_ETL_OUTPUT_TRIGGER_CHANNEL0
};

static const XMC_ERU_OGU_CONFIG_t button_event_detection_config =
{
  .service_request = XMC_ERU_OGU_SERVICE_REQUEST_ON_TRIGGER
};

__STATIC_INLINE void delay(uint32_t cycles)
{
  while(--cycles)
  {
    __NOP();
  }
}

void IRQ3_Handler(void)
{
  XMC_GPIO_ToggleOutput(LED);

  // DO something
  delay(SystemCoreClock);

  XMC_GPIO_ToggleOutput(LED);

}

int main(void)
{
  XMC_ERU_ETL_Init(ERU0_ETL1, &button_event_generator_config);
  XMC_ERU_OGU_Init(ERU0_OGU0, &button_event_detection_config);

  XMC_GPIO_CONFIG_t config;

  config.mode = XMC_GPIO_MODE_OUTPUT_PUSH_PULL;
  config.output_level = XMC_GPIO_OUTPUT_LEVEL_HIGH;
  XMC_GPIO_Init(LED, &config);

  config.mode = XMC_GPIO_MODE_INPUT_PULL_UP;
  XMC_GPIO_Init(POT, &config);

  NVIC_SetPriority(IRQ3_IRQn, 3U);
  NVIC_EnableIRQ(IRQ3_IRQn);

#if ENABLE_DEEP_SLEEP_MODE == 1
  SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;
#else
  SCB->SCR &= ~SCB_SCR_SLEEPDEEP_Msk;
#endif

  /* Enable sleep-on-exit feature */
  SCB->SCR |= SCB_SCR_SLEEPONEXIT_Msk;

  while(1)
  {
    __DSB();
    __WFI();
  }
}
