/**
 * @file main.c
 * @date 2021-01-08
 * @brief XMC1400 Bootkit TSE example
 *
 * TSE example uses ROM function to calculate internal temperature
 * 
 * @cond
 *****************************************************************************
 * XMClib v2.2.0 - XMC Peripheral Driver Library
 *
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 *****************************************************************************
 *
 * Change History
 * --------------
 *
 * 2015-11-09:
 *     - Initial version
 *
 * @endcond
 *
 */

#include <stdio.h>

#include "xmc_scu.h"
#include "xmc_gpio.h"
#include "xmc_uart.h"

#define TICKS_PER_SECOND 1000
#define LED1 P4_0

/* UART pins */
#define UART_TX P1_2
#define UART_RX P1_3

const XMC_SCU_CLOCK_CONFIG_t clock_config =
{
  .fdiv = 0,
  .idiv = 1,
  .pclk_src = XMC_SCU_CLOCK_PCLKSRC_DOUBLE_MCLK
};

void SystemCoreClockSetup(void)
{
  XMC_SCU_CLOCK_Init(&clock_config);

  SystemCoreClockUpdate();
}

const XMC_UART_CH_CONFIG_t uart_config =
{
  .data_bits = 8U,
  .stop_bits = 1U,
  .baudrate = 115200U
};

void SysTick_Handler(void)
{
  static uint32_t ticks = 0U;

  ++ticks;
  if (ticks == TICKS_PER_SECOND)
  {
    ticks = 0U;
    XMC_GPIO_ToggleOutput(LED1);

    printf("Chip temperature: %d\r\n", (unsigned int)XMC_SCU_CalcTemperature());
  }
}

int main(void)
{
  XMC_UART_CH_Init(XMC_UART0_CH1, &uart_config);

  XMC_GPIO_SetMode(UART_RX, XMC_GPIO_MODE_INPUT_TRISTATE);
  XMC_UART_CH_SetInputSource(XMC_UART0_CH1, XMC_UART_CH_INPUT_RXD, USIC0_C1_DX0_P1_3);

  XMC_UART_CH_Start(XMC_UART0_CH1);

  XMC_GPIO_SetMode(UART_TX, (XMC_GPIO_MODE_t)((int32_t)XMC_GPIO_MODE_OUTPUT_PUSH_PULL | (int32_t)P1_2_AF_U0C1_DOUT0));

  XMC_GPIO_SetMode(LED1, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  SysTick_Config(SystemCoreClock / TICKS_PER_SECOND);

  XMC_SCU_StartTempMeasurement();

  /* Placeholder for user application code. The while loop below can be replaced with user application code. */
  while(1U);
}
