/**
 * @file retarget_io.c
 * @date 2021-01-08
 * @brief XMC1400 Bootkit TSE example
 *
 * @cond
 *****************************************************************************
 * XMClib v2.2.0 - XMC Peripheral Driver Library
 *
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 *****************************************************************************
 *
 * Change History
 * --------------
 *
 * 2015-11-09:
 *     - Initial version
 *
 * 2021-01-08:
 *     - Fixed compilation for ARMClang
 * 
 * @endcond
 *
 */

#include <stdio.h>
#include "serial.h"

#if defined(__GNUC__) && !defined (__ARMCC_VERSION)

int _write(int fd, const void *buf, size_t count)
{
  (void)fd;

  for (size_t i = 0; i < count; ++i)
  {
    XMC_UART_CH_Transmit(SERIAL_UART, *(const uint8_t *)buf);
    buf++;
  }
  return count;
}

int _read(int fd, void *buf, size_t count)
{
  int char_cnt = 0;
  (void)fd;

  for (size_t i = 0; i < count; ++i)
  {
	if (ring_buffer_get(&serial_buffer, (uint8_t *)buf) != 0)
	{
	  break;
	}

    char_cnt++;

    /* Stop reading if CR (Ox0D) character is received */
    if (*(uint8_t *)buf == 0x0DU)
    {
      /* New line character (CR) received ? */
      *(uint8_t*)buf = '\n';           /* Yes, convert LF to '\n' char. */
      break;                           /* Stop loop and return received char(s) */
    }

    buf++;
  }

  return char_cnt;
}
#endif

#if defined(__CC_ARM) || defined(__ARMCC_VERSION)

/**
  Get a character from stdin
 
  \return     The next character from the input, or -1 on read error.
*/
int stdin_getchar (void) {
	uint16_t val;
	
  while ((XMC_UART_CH_GetStatusFlag(SERIAL_UART) & (XMC_UART_CH_STATUS_FLAG_ALTERNATIVE_RECEIVE_INDICATION |
                                                      XMC_UART_CH_STATUS_FLAG_RECEIVE_INDICATION)) == 0);
	val = XMC_UART_CH_GetReceivedData(SERIAL_UART);
	
	XMC_UART_CH_ClearStatusFlag(SERIAL_UART, XMC_UART_CH_STATUS_FLAG_ALTERNATIVE_RECEIVE_INDICATION |
                                             XMC_UART_CH_STATUS_FLAG_RECEIVE_INDICATION);
  return (val);
}

/**
  Put a character to the stdout
 
  \param[in]   ch  Character to output
  \return          The character written, or -1 on write error.
*/
int stdout_putchar (int ch) {
  XMC_UART_CH_Transmit(SERIAL_UART, ch);
  return (1);
}
#endif

#if defined(__ICCARM__)
size_t __write(int Handle, const unsigned char * Buf, size_t Bufsize)
{
  for (size_t i = 0; i < Bufsize; ++i)
  {
    XMC_UART_CH_Transmit(SERIAL_UART, Buf[i]);
  }
  return Bufsize;
}

int __read(int Handle, const unsigned char * Buf, size_t Bufsize)
{
  int char_cnt = 0;

  for (size_t i = 0; i < Bufsize; ++i)
  {
	if (ring_buffer_get(&serial_buffer, (uint8_t *)Buf) != 0)
	{
	  break;
	}

    char_cnt++;

    /* Stop reading if CR (Ox0D) character is received */
    if (*(uint8_t *)Buf == 0x0DU)
    {
      /* New line character (CR) received ? */
      *(uint8_t*)Buf = '\n';           /* Yes, convert LF to '\n' char. */
      break;                           /* Stop loop and return received char(s) */
    }

    Buf++;
  }

  return char_cnt;
}

#endif

