/*
 * Copyright (C) 2019 Infineon Technologies AG. All rights reserved.
 *
 * Infineon Technologies AG (Infineon) is supplying this software for use
 * with Infineon's microcontrollers.
 * This file can be freely distributed within development tools that are
 * supporting such microcontrollers.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS,
 * IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS
 * SOFTWARE. INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 */

/**
 * @file main.c
 * @date 02 Feb, 2015
 * @version 0.1.0
 *
 * @brief Dedicated Sequencing of Several Channels - Using the Queue Source
 *
 * This example shows how to convert multiple channels in a dedicated
 * sequence using the queue source. The queue FIFO is Channel 0, 3, 5, 1, 1.
 * Channel 3, 5, 1 and 1 are configured as refill so the queue will be fed
 * again with these channels after the first conversion. Channel 0 is
 * configured as no refill and to produce an interrupt after finishing its
 * conversion.
 * Channel 0 and 1 are configured with external trigger enabled. The CCU4
 * is used to trigger the queue periodically.
 *
 * In this example the KIT_XMC13_BOOT_001 is used. It includes a XMC1300
 * Microcontroller with 200KB Flash and a SEGGER J-Link Debugger.
 *
 * Hardware setup:
 * For download and debug, connect USB to the Boot Kit XMC1300 Board
 * (KIT_XMC13_BOOT_001) USB port.
 *
 * Download and start project.
 * While debugging, the analog values will be available in the array
 * "result".
 *
 * Following resources are used:
 * VADC group 0 channel 0
 * VADC group 0 channel 1
 * VADC group 0 channel 3
 * VADC group 0 channel 5
 * CCU4 Slice 0
 * Port P2.6 VADC input
 * Port P2.8 VADC input
 * Port P2.10 VADC input
 * Port P2.0 VADC input
 *
 * History
 *
 * Version 0.1.0 Initial  
 *
 */

/***************************************************************************
 * HEADER FILES
 **************************************************************************/
#include "xmc_vadc.h"
#include "xmc_ccu4.h"

/***************************************************************************
 * CONFIGURATION
 **************************************************************************/
/* VADC Global resources data configuration. No configuration needed,
 * standard values are used. */
const XMC_VADC_GLOBAL_CONFIG_t g_global_handle;

/* VADC group data configuration. No configuration needed, standard values
 * are used. */
const XMC_VADC_GROUP_CONFIG_t g_group_handle;

/* Data configuration for queue source. The trigger input A is used to
 * trigger the conversion on any edge (see also "Digital Connections in the
 * XMC4500" in the Reference Manual). */
const XMC_VADC_QUEUE_CONFIG_t g_queue_handle = {
    .trigger_signal = XMC_VADC_REQ_TR_A,
    .trigger_edge = XMC_VADC_TRIGGER_EDGE_ANY,
    .external_trigger = 1 };

/* Channel data configuration. Channels do NOT use alias feature and use
 * desired result registers. */
const XMC_VADC_CHANNEL_CONFIG_t g_g0_ch0_handle = {
    .alias_channel = XMC_VADC_CHANNEL_ALIAS_DISABLED,
    .result_reg_number = 0, };
const XMC_VADC_CHANNEL_CONFIG_t g_g0_ch1_handle = {
    .alias_channel = XMC_VADC_CHANNEL_ALIAS_DISABLED,
    .result_reg_number = 1, };
const XMC_VADC_CHANNEL_CONFIG_t g_g0_ch3_handle = {
    .alias_channel = XMC_VADC_CHANNEL_ALIAS_DISABLED,
    .result_reg_number = 3, };
const XMC_VADC_CHANNEL_CONFIG_t g_g0_ch5_handle = {
    .alias_channel = XMC_VADC_CHANNEL_ALIAS_DISABLED,
    .result_reg_number = 5, };

/* Queue entry configuration. For each entry a channel is configured. Entry
 * 1 to 4 are configured to be refilled automaticfally. Entry 0 and 3 can
 * be triggered externally and entry 0 generates an event after
 * conversion. */
const XMC_VADC_QUEUE_ENTRY_t g_queue_entry_0_handle = {
    .channel_num = 0,
    .refill_needed = 0,
    .external_trigger = 1,
    .generate_interrupt = 1, };
const XMC_VADC_QUEUE_ENTRY_t g_queue_entry_1_handle = {
    .channel_num = 3,
    .refill_needed = 1,
    .external_trigger = 0,
    .generate_interrupt = 0, };
const XMC_VADC_QUEUE_ENTRY_t g_queue_entry_2_handle = {
    .channel_num = 5,
    .refill_needed = 1,
    .external_trigger = 0,
    .generate_interrupt = 0, };
const XMC_VADC_QUEUE_ENTRY_t g_queue_entry_3_handle = {
    .channel_num = 1,
    .refill_needed = 1,
    .external_trigger = 1,
    .generate_interrupt = 0, };
const XMC_VADC_QUEUE_ENTRY_t g_queue_entry_4_handle = {
    .channel_num = 1,
    .refill_needed = 1,
    .external_trigger = 0,
    .generate_interrupt = 0, };

/* Timer (CCU4) configuration. The CCU4 is configured to use edge aligned
 * mode and a prescaler divider value of 7. */
const XMC_CCU4_SLICE_COMPARE_CONFIG_t g_timer_object = {
    .timer_mode = XMC_CCU4_SLICE_TIMER_COUNT_MODE_EA,
    .prescaler_initval = (uint32_t) 7, };

/***************************************************************************
 * GLOBAL DATA
 **************************************************************************/
XMC_VADC_RESULT_SIZE_t result[16] = { 0 };

/***************************************************************************
 * Function declarations
 **************************************************************************/
int main(void) {

    /* Clock is already initialized by startup code. */

    /* ****************************** VADC ****************************** */
    /* Provide clock to VADC and initialize the VADC global registers. */
    XMC_VADC_GLOBAL_Init(VADC, &g_global_handle);

    /* Initialize the conversion kernel. */
    XMC_VADC_GROUP_Init(VADC_G0, &g_group_handle);

    /* Set VADC group to normal operation mode (VADC kernel). */
    XMC_VADC_GROUP_SetPowerMode(VADC_G0, XMC_VADC_GROUP_POWERMODE_NORMAL);

    /* Calibrate the VADC. Make sure you do this after all used VADC groups
     * are set to normal operation mode. */
    XMC_VADC_GLOBAL_StartupCalibration(VADC);

    /* Initialize the queue source hardware. The gating mode is set to
     * ignore to pass external triggers unconditionally. */
    XMC_VADC_GROUP_QueueInit(VADC_G0, &g_queue_handle);

    /* Initialize the channel units. */
    XMC_VADC_GROUP_ChannelInit(VADC_G0, 0, &g_g0_ch0_handle);
    XMC_VADC_GROUP_ChannelInit(VADC_G0, 1, &g_g0_ch1_handle);
    XMC_VADC_GROUP_ChannelInit(VADC_G0, 3, &g_g0_ch3_handle);
    XMC_VADC_GROUP_ChannelInit(VADC_G0, 5, &g_g0_ch5_handle);

    /* Add a queue node to the tail of the queue. */
    XMC_VADC_GROUP_QueueInsertChannel(VADC_G0, g_queue_entry_0_handle);
    XMC_VADC_GROUP_QueueInsertChannel(VADC_G0, g_queue_entry_1_handle);
    XMC_VADC_GROUP_QueueInsertChannel(VADC_G0, g_queue_entry_2_handle);
    XMC_VADC_GROUP_QueueInsertChannel(VADC_G0, g_queue_entry_3_handle);
    XMC_VADC_GROUP_QueueInsertChannel(VADC_G0, g_queue_entry_4_handle);

    /* ****************************** CCU4 ****************************** */
    /* Initialize CCU4 global resources. */
    XMC_CCU4_Init(CCU40, XMC_CCU4_SLICE_MCMS_ACTION_TRANSFER_PR_CR);

    /* Start the prescaler and restore clocks to the timer slices. */
    XMC_CCU4_StartPrescaler(CCU40);

    /* Configure CC4 Slice in compare mode. */
    XMC_CCU4_SLICE_CompareInit(CCU40_CC40, &g_timer_object);

    /* Program period match value of the timer. */
    XMC_CCU4_SLICE_SetTimerPeriodMatch(CCU40_CC40, 0xFFFE);

    /* Transfer value from shadow timer registers to actual timer
     * registers. */
    XMC_CCU4_EnableShadowTransfer(CCU40, CCU4_GCSS_S0SE_Msk);

    /* Enable period/compare match event. */
    XMC_CCU4_SLICE_EnableEvent(CCU40_CC40,
            XMC_CCU4_SLICE_IRQ_ID_PERIOD_MATCH);

    /* Bind capcom event to an NVIC node (see also "Digital Connections in
     * the XMC4500" in the Reference Manual). */
    XMC_CCU4_SLICE_SetInterruptNode(CCU40_CC40,
            XMC_CCU4_SLICE_IRQ_ID_PERIOD_MATCH, XMC_CCU4_SLICE_SR_ID_2);

    /* Enable the slice timer clock. */
    XMC_CCU4_EnableClock(CCU40, 0);

    /* Start the timer counting operation. */
    XMC_CCU4_SLICE_StartTimer(CCU40_CC40);

    /* ****************************** NVIC ****************************** */
    /* Enable NVIC nodes */
    NVIC_EnableIRQ(IRQ17_IRQn);

    /* ****************************** Loop ****************************** */
    while (1U) {
        /* Retrieve result from result register. */
        result[1] = XMC_VADC_GROUP_GetResult(VADC_G0, 1);
        result[3] = XMC_VADC_GROUP_GetResult(VADC_G0, 3);
        result[5] = XMC_VADC_GROUP_GetResult(VADC_G0, 5);
    }
}

/* Interrupt Handler -  This function handles the SR0 of the Group 0.
 * It is activated with XMC_VADC_GROUP_ScanInit(). */
void IRQ17_Handler(void) {
    /* Retrieve result from result register. */
    result[0] = XMC_VADC_GROUP_GetResult(VADC_G0, 0);
}
