/*
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.                        
 *                                             
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *                                                                              
 * To improve the quality of the software, users are encouraged to share 
 * modifications, enhancements or bug fixes with Infineon Technologies AG 
 * at XMCSupport@infineon.com.
 *
 */

/**
 * @file I2C.c
 * @date 16 Dec., 2019
 * @version 2.10
 *
 * @brief I2C Driver for Infineon XMC devices
 *
 * History
 *
 * Version 2.10 Added IRQ priority and HDEL settings
 *
 * Version 2.9 Fix compiler warnings
 *             Fix BUS_CLEAR
 *             Enable digital filters on inputs
 *             Fix I2C_GetDataCount()
 *             I2C_MasterTransmit(), I2C_MasterReceive() added busy bus check
 * 
 * Version 2.8 Conditional compiling based on RTE_Drivers_I2C
 * 
 * Version 2.7 Fix compiler portability
 *             Avoid XMC_USIC_CH_RXFIFO_SetSizeTriggerLimit gets negative value of limit parameter
 *             Fix I2C_PowerControl for ARM_POWER_OFF
 *
 * Version 2.6 Fixed IRQ handlers names
 *
 * Version 2.5 Fixed IRQ handlers names
 *
 * Version 2.4 Adapted to xmc1400 devices<br>
 *
 * Version 2.3 Added missing events at the end of slave operation. <br>
 *
 * Version 2.2 Added fixed to prevent race conditions 
 *               and Initialize/Uninitialize, Power Control 
 *               guidelines related modifications <br>
 *
 * Version 1.0 Initial version<br>
 */

#include "I2C.h"

#if defined(RTE_Drivers_I2C)

#if (!RTE_I2C0 && !RTE_I2C1 && !RTE_I2C2 && !RTE_I2C3 && !RTE_I2C4 && !RTE_I2C5)
#error "I2C not configured in RTE_Device.h!"
#endif

#define ARM_I2C_DRV_VERSION ARM_DRIVER_VERSION_MAJOR_MINOR(2,10)   /* driver version */

#define I2C_MODE_MASTER 1
#define I2C_MODE_SLAVE  0 

#define I2C_DIR_WRITE   0
#define I2C_DIR_READ    1

// Driver Version
static const ARM_DRIVER_VERSION DriverVersion = 
{
  ARM_I2C_API_VERSION,
  ARM_I2C_DRV_VERSION
};

// Default I2C initialization
static const XMC_I2C_CH_CONFIG_t i2c_default_config =
{
  .baudrate = 100000U,
  .address =  0U,
  .normal_divider_mode = true
};

#if (defined(RTE_Drivers_I2C) && \
     (RTE_I2C0 == 0) && \
     (RTE_I2C1 == 0) && \
     (RTE_I2C2 == 0) && \
     (RTE_I2C3 == 0) && \
     (RTE_I2C4 == 0) && \
     (RTE_I2C5 == 0))
#error "I2C not configured in RTE_Device.h!"
#endif

static const ARM_I2C_CAPABILITIES DriverCapabilities =
{
  0   //< supports 10-bit addressing
};


/* I2C0 */
#if (RTE_I2C0 != 0)

static volatile I2C_INFO I2C0_Info;
static XMC_GPIO_CONFIG_t I2C0_sda_conf; 
static XMC_GPIO_CONFIG_t I2C0_scl_conf; 

/* I2C0 Resources */
static const I2C_RESOURCES I2C0_Resources = 
{
  {RTE_I2C0_TX_PORT},
  RTE_I2C0_TX_AF,
  {RTE_I2C0_RX_PORT},
  &I2C0_sda_conf,
  RTE_I2C0_RX_INPUT,
  {RTE_I2C0_CLK_OUTPUT_PORT},
  RTE_I2C0_CLK_AF,
  {RTE_I2C0_CLK_INPUT_PORT},
  &I2C0_scl_conf,
  RTE_I2C0_CLK_INPUT,
  XMC_I2C0_CH0,
  (IRQn_Type)USIC0_0_IRQn,
  RTE_I2C0_IRQ_PRIORITY,
  RTE_I2C0_HDEL,
  RTE_I2C0_TX_FIFO_SIZE_NUM,
  RTE_I2C0_TX_FIFO_SIZE,
  RTE_I2C0_RX_FIFO_SIZE_NUM,
  RTE_I2C0_RX_FIFO_SIZE,
  &I2C0_Info
};
#endif

/* I2C1 */
#if (RTE_I2C1 != 0)

static volatile I2C_INFO I2C1_Info;
static XMC_GPIO_CONFIG_t I2C1_sda_conf; 
static XMC_GPIO_CONFIG_t I2C1_scl_conf; 

/* I2C1 Resources */
static const I2C_RESOURCES I2C1_Resources = 
{
  {RTE_I2C1_TX_PORT},
  RTE_I2C1_TX_AF,
  {RTE_I2C1_RX_PORT},
  &I2C1_sda_conf,
  RTE_I2C1_RX_INPUT,
  {RTE_I2C1_CLK_OUTPUT_PORT},
  RTE_I2C1_CLK_AF,
  {RTE_I2C1_CLK_INPUT_PORT},
  &I2C1_scl_conf,
  RTE_I2C1_CLK_INPUT,
  XMC_I2C0_CH1,
  (IRQn_Type)USIC0_1_IRQn,
  RTE_I2C1_IRQ_PRIORITY,
  RTE_I2C1_HDEL,
  RTE_I2C1_TX_FIFO_SIZE_NUM,
  RTE_I2C1_TX_FIFO_SIZE,
  RTE_I2C1_RX_FIFO_SIZE_NUM,
  RTE_I2C1_RX_FIFO_SIZE,
  &I2C1_Info
};
#endif

/* I2C2 */
#if (RTE_I2C2 != 0)

static volatile I2C_INFO I2C2_Info;
static XMC_GPIO_CONFIG_t I2C2_sda_conf; 
static XMC_GPIO_CONFIG_t I2C2_scl_conf; 

/* I2C2 Resources */
static const I2C_RESOURCES I2C2_Resources = 
{
  {RTE_I2C2_TX_PORT},
  RTE_I2C2_TX_AF,
  {RTE_I2C2_RX_PORT},
  &I2C2_sda_conf,
  RTE_I2C2_RX_INPUT,
  {RTE_I2C2_CLK_OUTPUT_PORT},
  RTE_I2C2_CLK_AF,
  {RTE_I2C2_CLK_INPUT_PORT},
  &I2C2_scl_conf,
  RTE_I2C2_CLK_INPUT,
  XMC_I2C1_CH0,
  (IRQn_Type)USIC1_0_IRQn,
  RTE_I2C2_IRQ_PRIORITY,
  RTE_I2C2_HDEL,
  RTE_I2C2_TX_FIFO_SIZE_NUM,
  RTE_I2C2_TX_FIFO_SIZE,
  RTE_I2C2_RX_FIFO_SIZE_NUM,
  RTE_I2C2_RX_FIFO_SIZE,
  &I2C2_Info
};
#endif

/* I2C3 */
#if (RTE_I2C3 != 0)

static volatile I2C_INFO I2C3_Info;
static XMC_GPIO_CONFIG_t I2C3_sda_conf; 
static XMC_GPIO_CONFIG_t I2C3_scl_conf; 

/* I2C3 Resources */
static const I2C_RESOURCES I2C3_Resources = 
{
  {RTE_I2C3_TX_PORT},
  RTE_I2C3_TX_AF,
  {RTE_I2C3_RX_PORT},
  &I2C3_sda_conf,
  RTE_I2C3_RX_INPUT,
  {RTE_I2C3_CLK_OUTPUT_PORT},
  RTE_I2C3_CLK_AF,
  {RTE_I2C3_CLK_INPUT_PORT},
  &I2C3_scl_conf,
  RTE_I2C3_CLK_INPUT,
  XMC_I2C1_CH1,
  (IRQn_Type)USIC1_1_IRQn,
  RTE_I2C3_IRQ_PRIORITY,
  RTE_I2C3_HDEL,
  RTE_I2C3_TX_FIFO_SIZE_NUM,
  RTE_I2C3_TX_FIFO_SIZE,
  RTE_I2C3_RX_FIFO_SIZE_NUM,
  RTE_I2C3_RX_FIFO_SIZE,
  &I2C3_Info
};
#endif


/* I2C4 */
#if (RTE_I2C4 != 0)

static volatile I2C_INFO I2C4_Info;
static XMC_GPIO_CONFIG_t I2C4_sda_conf; 
static XMC_GPIO_CONFIG_t I2C4_scl_conf; 

/* I2C4 Resources */
static const I2C_RESOURCES I2C4_Resources = 
{
  {RTE_I2C4_TX_PORT},
  RTE_I2C4_TX_AF,
  {RTE_I2C4_RX_PORT},
  &I2C4_sda_conf,
  RTE_I2C4_RX_INPUT,
  {RTE_I2C4_CLK_OUTPUT_PORT},
  RTE_I2C4_CLK_AF,
  {RTE_I2C4_CLK_INPUT_PORT},
  &I2C4_scl_conf,
  RTE_I2C4_CLK_INPUT,
  XMC_I2C2_CH0,
  (IRQn_Type)USIC2_0_IRQn,
  RTE_I2C4_IRQ_PRIORITY,
  RTE_I2C4_HDEL,
  RTE_I2C4_TX_FIFO_SIZE_NUM,
  RTE_I2C4_TX_FIFO_SIZE,
  RTE_I2C4_RX_FIFO_SIZE_NUM,
  RTE_I2C4_RX_FIFO_SIZE,
  &I2C4_Info
};
#endif


/* I2C5 */
#if (RTE_I2C5 != 0)

static volatile I2C_INFO I2C5_Info;
static XMC_GPIO_CONFIG_t I2C5_sda_conf; 
static XMC_GPIO_CONFIG_t I2C5_scl_conf; 

/* I2C4 Resources */
static const I2C_RESOURCES I2C5_Resources = 
{
  {RTE_I2C5_TX_PORT},
  RTE_I2C5_TX_AF,
  {RTE_I2C5_RX_PORT},
  &I2C5_sda_conf,
  RTE_I2C5_RX_INPUT,
  {RTE_I2C5_CLK_OUTPUT_PORT},
  RTE_I2C5_CLK_AF,
  {RTE_I2C5_CLK_INPUT_PORT},
  &I2C5_scl_conf,
  RTE_I2C5_CLK_INPUT,
  XMC_I2C2_CH1,
  (IRQn_Type)USIC2_1_IRQn,
  RTE_I2C5_IRQ_PRIORITY,
  RTE_I2C5_HDEL,
  RTE_I2C5_TX_FIFO_SIZE_NUM,
  RTE_I2C5_TX_FIFO_SIZE,
  RTE_I2C5_RX_FIFO_SIZE_NUM,
  RTE_I2C5_RX_FIFO_SIZE,
  &I2C5_Info
};

#endif


/* I2C Resources */
static const I2C_RESOURCES *const i2c_resources[6] = 
{
#if (RTE_I2C0 != 0)
  &I2C0_Resources,
#else
  NULL,
#endif
#if (RTE_I2C1 != 0)
  &I2C1_Resources,
#else
  NULL,
#endif
#if (RTE_I2C2 != 0)
  &I2C2_Resources,
#else
  NULL,
#endif
#if (RTE_I2C3 != 0)
  &I2C3_Resources,
#else
  NULL,
#endif
#if (RTE_I2C4 != 0)
  &I2C4_Resources,
#else
  NULL,
#endif
#if (RTE_I2C5 != 0)
  &I2C5_Resources,
#else
  NULL,
#endif
};

/**
  \fn          ARM_DRV_VERSION I2CX_GetVersion (void)
  \brief       Get driver version.
  \return      \ref ARM_DRV_VERSION
*/
static ARM_DRIVER_VERSION I2CX_GetVersion(void) 
{
  return DriverVersion;
}


/**
  \fn          ARM_I2C_CAPABILITIES I2C_GetCapabilities (I2C_RESOURCES  *i2c)
  \brief       Get driver capabilities.
  \param[in]   i2c    Pointer to I2C resources
  \return      \ref I2C_CAPABILITIES
*/
__STATIC_INLINE ARM_I2C_CAPABILITIES I2C_GetCapabilities(I2C_RESOURCES *const i2c) 
{
  (void)i2c;
  return DriverCapabilities;
}

#if (RTE_I2C0 != 0)
static ARM_I2C_CAPABILITIES I2C0_GetCapabilities(void)
{
  return I2C_GetCapabilities(i2c_resources[0]);
}
#endif

#if (RTE_I2C1 != 0)
static ARM_I2C_CAPABILITIES I2C1_GetCapabilities(void) 
{
  return I2C_GetCapabilities(i2c_resources[1]);
}
#endif

#if (RTE_I2C2 != 0)
static ARM_I2C_CAPABILITIES I2C2_GetCapabilities(void) 
{
  return I2C_GetCapabilities(i2c_resources[2]);
}
#endif

#if (RTE_I2C3 != 0)
static ARM_I2C_CAPABILITIES I2C3_GetCapabilities(void) 
{
  return I2C_GetCapabilities(i2c_resources[3]);
}
#endif

#if (RTE_I2C4 != 0)
static ARM_I2C_CAPABILITIES I2C4_GetCapabilities(void)
{
  return I2C_GetCapabilities(i2c_resources[4]);
}
#endif

#if (RTE_I2C5 != 0)
static ARM_I2C_CAPABILITIES I2C5_GetCapabilities(void) 
{
  return I2C_GetCapabilities(i2c_resources[5]);
}
#endif

/**
  \fn          ARM_I2C_STATUS I2C_Initialize (ARM_I2C_SignalEvent_t cb_event, I2C_RESOURCES  *i2c)
  \brief       Initialize I2C Interface.
  \param[in]   cb_event    Pointer to \ref ARM_I2C_SignalEvent
  \param[in]   i2c    Pointer to I2C resources
  \return      \ref ARM_I2C_STATUS
*/
static int32_t I2C_Initialize(ARM_I2C_SignalEvent_t cb_event, I2C_RESOURCES *const i2c)
{
  if (((i2c->info->flags) & I2C_FLAG_INITIALIZED) == 0)
  {
    // Initialize I2C Run-Time Resources
    i2c->info->cb_event = cb_event;                                        
    
    /* Clear driver status */
    i2c->info->status.arbitration_lost = 0;
    i2c->info->status.busy = 0;
    i2c->info->status.bus_error = 0;
    i2c->info->status.direction = 0;
    i2c->info->pending = false;
    i2c->info->status.general_call=0;
    i2c->info->status.mode = I2C_MODE_MASTER;
    i2c->info->flags |= I2C_FLAG_INITIALIZED;
  }
  
  return ARM_DRIVER_OK;
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_Initialize(ARM_I2C_SignalEvent_t cb_event) 
{
  return I2C_Initialize(cb_event, i2c_resources[0]);
}
#endif

#if (RTE_I2C1 != 0)
static int32_t I2C1_Initialize(ARM_I2C_SignalEvent_t cb_event) 
{
  return I2C_Initialize(cb_event, i2c_resources[1]);
}
#endif

#if (RTE_I2C2 != 0)
static int32_t I2C2_Initialize(ARM_I2C_SignalEvent_t cb_event) 
{
  return I2C_Initialize(cb_event, i2c_resources[2]);
}
#endif

#if (RTE_I2C3 != 0)
static int32_t I2C3_Initialize(ARM_I2C_SignalEvent_t cb_event) 
{
  return I2C_Initialize(cb_event, i2c_resources[3]);
}
#endif

#if (RTE_I2C4 != 0)
static int32_t I2C4_Initialize(ARM_I2C_SignalEvent_t cb_event) 
{
  return I2C_Initialize(cb_event, i2c_resources[4]);
}
#endif

#if (RTE_I2C5 != 0)
static int32_t I2C5_Initialize(ARM_I2C_SignalEvent_t cb_event) 
{
  return I2C_Initialize(cb_event, i2c_resources[5]);
}
#endif


  /**
  \fn          int32_t I2C_Uninitialize (I2C_RESOURCES *i2c)
  \brief       De-initialize I2C Interface.
  \param[in]   i2c     Pointer to I2C resources
  \return      \ref execution_status
*/
static int32_t I2C_Uninitialize (I2C_RESOURCES *const i2c) 
{
  // Reset I2C status flags
  i2c->info->flags = 0;
  return ARM_DRIVER_OK;
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_Uninitialize(void) 
{
  return I2C_Uninitialize(i2c_resources[0]);
}
#endif  
#if (RTE_I2C1 != 0)
static int32_t I2C1_Uninitialize(void) 
{
  return I2C_Uninitialize(i2c_resources[1]);
}
#endif
#if (RTE_I2C2 != 0)
static int32_t I2C2_Uninitialize(void) 
{
  return I2C_Uninitialize(i2c_resources[2]);
}
#endif
#if (RTE_I2C3 != 0)
static int32_t I2C3_Uninitialize(void) 
{
  return I2C_Uninitialize(i2c_resources[3]);
}
#endif
#if (RTE_I2C4 != 0)
static int32_t I2C4_Uninitialize(void) 
{
  return I2C_Uninitialize(i2c_resources[4]);
}
#endif
#if (RTE_I2C5 != 0)
static int32_t I2C5_Uninitialize(void) 
{
  return I2C_Uninitialize(i2c_resources[5]);
}
#endif
/**
  \fn          ARM_I2C_STATUS I2C_PowerControl (ARM_POWER_STATE  state,
                                                  I2C_RESOURCES   *i2c)
  \brief       Controls I2C Interface Power.
  \param[in]   state    Power state
  \param[in]   i2c Pointer to I2C resources
  \return      \ref I2C_STATUS
*/
static int32_t I2C_PowerControl(ARM_POWER_STATE state, I2C_RESOURCES *const i2c)
{
  if (state == ARM_POWER_FULL) 
  {
    if ((i2c->info->flags & I2C_FLAG_INITIALIZED) == 0U) 
    {
      return ARM_DRIVER_ERROR;
    }
    if ((i2c->info->flags & I2C_FLAG_POWERED)!= 0U) 
    {
      return ARM_DRIVER_OK;
    }
    
    if (i2c->tx_fifo_size_num > i2c->rx_fifo_size_num)
    {
      i2c->info->tx_fifo_pointer = (((uint32_t)(i2c->i2c) & 0x200U) != 0) ? 32 : 0;
      i2c->info->rx_fifo_pointer = i2c->info->tx_fifo_pointer + i2c->tx_fifo_size_num;
    }
    else
    {
      i2c->info->rx_fifo_pointer = (((uint32_t)(i2c->i2c) & 0x200U) != 0) ? 32 : 0;
      i2c->info->tx_fifo_pointer = i2c->info->rx_fifo_pointer + i2c->rx_fifo_size_num;
    }
    
    XMC_I2C_CH_Init(i2c->i2c, &i2c_default_config);
    XMC_USIC_CH_TXFIFO_Configure(i2c->i2c, i2c->info->tx_fifo_pointer, (XMC_USIC_CH_FIFO_SIZE_t)i2c->tx_fifo_size_reg, 1U);
    XMC_I2C_CH_SetInputSource(i2c->i2c, XMC_I2C_CH_INPUT_SDA, i2c->sda_pin_input);
    XMC_I2C_CH_SetInputSource(i2c->i2c, XMC_I2C_CH_INPUT_SCL, i2c->scl_pin_input);

    XMC_USIC_CH_EnableInputDigitalFilter(i2c->i2c, (XMC_USIC_CH_INPUT_t)XMC_I2C_CH_INPUT_SDA);
    XMC_USIC_CH_EnableInputSync(i2c->i2c, (XMC_USIC_CH_INPUT_t)XMC_I2C_CH_INPUT_SDA);
    XMC_USIC_CH_EnableInputDigitalFilter(i2c->i2c, (XMC_USIC_CH_INPUT_t)XMC_I2C_CH_INPUT_SCL);
    XMC_USIC_CH_EnableInputSync(i2c->i2c, (XMC_USIC_CH_INPUT_t)XMC_I2C_CH_INPUT_SCL);

    XMC_I2C_CH_SetHoldDelay(i2c->i2c, i2c->hdel);

    XMC_I2C_CH_Start(i2c->i2c);
  
    // Configure pin  
    i2c->pin_sda_config->mode = (XMC_GPIO_MODE_t)(XMC_GPIO_MODE_OUTPUT_OPEN_DRAIN | i2c->sda_pin_af_output);
    i2c->pin_sda_config->output_strength = XMC_GPIO_OUTPUT_STRENGTH_STRONG_MEDIUM_EDGE;
    XMC_GPIO_Init(i2c->sda_tx_port.port,i2c->sda_tx_port.pin, i2c->pin_sda_config);

    i2c->pin_scl_config->mode = (XMC_GPIO_MODE_t)(XMC_GPIO_MODE_OUTPUT_OPEN_DRAIN | i2c->scl_pin_af_output);
    i2c->pin_scl_config->output_strength = XMC_GPIO_OUTPUT_STRENGTH_STRONG_MEDIUM_EDGE;
    XMC_GPIO_Init(i2c->scl_output_port.port,i2c->scl_output_port.pin, i2c->pin_scl_config);                                                       

    NVIC_ClearPendingIRQ(i2c->irq_num);
    
#if(UC_FAMILY == XMC4)
    NVIC_SetPriority(i2c->irq_num, NVIC_EncodePriority(NVIC_GetPriorityGrouping(), i2c->irq_priority, 0U)); 
#else
    NVIC_SetPriority(i2c->irq_num, i2c->irq_priority); 
#endif

#if (UC_SERIES == XMC14)    
    XMC_SCU_SetInterruptControl(i2c->irq_num,(XMC_SCU_IRQCTRL_t)(((i2c->irq_num) << 8U) | 3U));
#endif

    NVIC_EnableIRQ(i2c->irq_num);

    i2c->info->flags |= I2C_FLAG_POWERED;
  } 
  else if(state == ARM_POWER_OFF) 
  {
    NVIC_DisableIRQ(i2c->irq_num);

    // Disable peripheral, terminates any pending data transfers
    while (XMC_I2C_CH_Stop(i2c->i2c) == XMC_I2C_CH_STATUS_BUSY)
    {
      ;
    }

    XMC_USIC_CH_TXFIFO_Flush(i2c->i2c); 
    XMC_USIC_CH_RXFIFO_Flush(i2c->i2c); 
    XMC_USIC_CH_TXFIFO_Configure(i2c->i2c, 0, XMC_USIC_CH_FIFO_DISABLED, 0U); 
    XMC_USIC_CH_RXFIFO_Configure(i2c->i2c, 0, XMC_USIC_CH_FIFO_DISABLED, 0U); 
    XMC_USIC_CH_Disable(i2c->i2c);
    i2c->pin_sda_config->mode = (XMC_GPIO_MODE_t)(XMC_GPIO_MODE_INPUT_TRISTATE);
    i2c->pin_scl_config->mode = (XMC_GPIO_MODE_t)(XMC_GPIO_MODE_INPUT_TRISTATE);

    XMC_GPIO_Init(i2c->sda_tx_port.port,     i2c->sda_tx_port.pin,     i2c->pin_sda_config);                                           
    XMC_GPIO_Init(i2c->scl_output_port.port, i2c->scl_output_port.pin, i2c->pin_scl_config);                                                       

    i2c->info->flags &= (uint8_t)~I2C_FLAG_POWERED;
  } 
  else 
  {
    return ARM_DRIVER_ERROR_UNSUPPORTED;
  }

  return ARM_DRIVER_OK;
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_PowerControl(ARM_POWER_STATE state) 
{
  return I2C_PowerControl(state, i2c_resources[0]);
}
#endif
#if (RTE_I2C1 != 0)
static int32_t I2C1_PowerControl(ARM_POWER_STATE state) 
{
  return I2C_PowerControl(state, i2c_resources[1]);
}
#endif
#if (RTE_I2C2 != 0)
static int32_t I2C2_PowerControl(ARM_POWER_STATE state) 
{
  return I2C_PowerControl(state, i2c_resources[2]);
}
#endif
#if (RTE_I2C3 != 0)
static int32_t I2C3_PowerControl(ARM_POWER_STATE state) 
{
  return I2C_PowerControl(state, i2c_resources[3]);
}
#endif
#if (RTE_I2C4 != 0)
static int32_t I2C4_PowerControl(ARM_POWER_STATE state) 
{
  return I2C_PowerControl(state, i2c_resources[4]);
}
#endif
#if (RTE_I2C5 != 0)
static int32_t I2C5_PowerControl(ARM_POWER_STATE state) 
{
  return I2C_PowerControl(state, i2c_resources[5]);
}
#endif
/**
  \fn          int32_t I2C_MasterTransmit (uint32_t       addr,
                                            const uint8_t *data,
                                            uint32_t       num,
                                            bool           xfer_pending,
                                            I2C_RESOURCES *i2c)
  \brief       Start transmitting data as I2C Master.
  \param[in]   addr          Slave address (7-bit)
  \param[in]   data          Pointer to buffer with data to transmit to I2C Slave
  \param[in]   num           Number of data bytes to transmit
  \param[in]   xfer_pending  Transfer operation is pending - Stop condition will not be generated
  \param[in]   i2c           Pointer to I2C resources
  \return      \ref execution_status
*/
static int32_t I2C_MasterTransmit(uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending, I2C_RESOURCES *const i2c) 
{
  uint8_t SRno;
  
  if ((data == NULL) || (num == 0)) {
    // Invalid parameters
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((i2c->info->flags & I2C_FLAG_POWERED) == 0)
  {
    // I2C is not powered 
    return ARM_DRIVER_ERROR;
  }

  if (i2c->info->status.busy || (XMC_GPIO_GetInput(i2c->sda_rx_port.port, i2c->sda_rx_port.pin) == 0))
  {
    /* Transfer operation in progress, or Slave stalled */
    return ARM_DRIVER_ERROR_BUSY;
  }
 
  XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_ERROR |
                                       XMC_I2C_CH_STATUS_FLAG_STOP_CONDITION_RECEIVED |
                                       XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED |
                                       XMC_I2C_CH_STATUS_FLAG_ARBITRATION_LOST |
                                       XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED);
  
  /* Set control variables */
  i2c->info->sla = addr;
  i2c->info->xfer.tx_buf            = (uint8_t *)data;
  i2c->info->xfer.tx_num            = num;
  i2c->info->xfer.tx_cnt            = 0;

  /* Update driver status */
  i2c->info->status.busy             = 1;
  i2c->info->status.mode             = I2C_MODE_MASTER;
  i2c->info->status.direction        = I2C_DIR_WRITE;
  i2c->info->status.arbitration_lost = 0;
  i2c->info->status.bus_error        = 0;

  SRno = (((uint32_t)(i2c->i2c) & 0x200U) != 0) ? 1 : 0;
#if (UC_SERIES == XMC14)
  SRno += (((uint32_t)(i2c->i2c) & 0x4000U) != 0) ? 1 : 0;
#endif

  XMC_USIC_CH_TXFIFO_Flush(i2c->i2c); 
  
  XMC_I2C_CH_EnableEvent(i2c->i2c, XMC_I2C_CH_EVENT_ACK |
                                   XMC_I2C_CH_EVENT_STOP_CONDITION_RECEIVED |
                                   XMC_I2C_CH_EVENT_NACK |
                                   XMC_I2C_CH_EVENT_ARBITRATION_LOST |
                                   XMC_I2C_CH_EVENT_ERROR);
  
  XMC_I2C_CH_SetInterruptNodePointer(i2c->i2c, SRno);
  
  if(i2c->info->pending == false)
  {  
    /* send slave address and command to write */
    XMC_I2C_CH_MasterStart(i2c->i2c, addr, XMC_I2C_CH_CMD_WRITE);
  }
  else
  {
    XMC_I2C_CH_MasterRepeatedStart(i2c->i2c, addr, XMC_I2C_CH_CMD_WRITE);
  }
  
  i2c->info->pending = xfer_pending;
  
  return ARM_DRIVER_OK;
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_MasterTransmit(uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending) 
{
  return I2C_MasterTransmit(addr, data, num, xfer_pending, i2c_resources[0]);
}
#endif

#if (RTE_I2C1 != 0)
static int32_t I2C1_MasterTransmit(uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending) 
{
  return I2C_MasterTransmit(addr, data, num, xfer_pending, i2c_resources[1]);
}
#endif

#if (RTE_I2C2 != 0)
static int32_t I2C2_MasterTransmit(uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending)
{
  return I2C_MasterTransmit(addr, data, num, xfer_pending, i2c_resources[2]);
}
#endif

#if (RTE_I2C3 != 0)
static int32_t I2C3_MasterTransmit(uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending) 
{
  return I2C_MasterTransmit(addr, data, num, xfer_pending, i2c_resources[3]);
}
#endif

#if (RTE_I2C4 != 0)
static int32_t I2C4_MasterTransmit(uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending) 
{
  return I2C_MasterTransmit(addr, data, num, xfer_pending, i2c_resources[4]);
}
#endif

#if (RTE_I2C5 != 0)
static int32_t I2C5_MasterTransmit(uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending) 
{
  return I2C_MasterTransmit(addr, data, num, xfer_pending, i2c_resources[5]);
}
#endif


/**
  \fn          int32_t I2C_MasterReceive (uint32_t       addr,
                                           uint8_t       *data,
                                           uint32_t       num,
                                           bool           xfer_pending,
                                           I2C_RESOURCES *i2c)
  \brief       Start receiving data as I2C Master.
  \param[in]   addr          Slave address (7-bit or 10-bit)
  \param[out]  data          Pointer to buffer for data to receive from I2C Slave
  \param[in]   num           Number of data bytes to receive
  \param[in]   xfer_pending  Transfer operation is pending - Stop condition will not be generated
  \param[in]   i2c           Pointer to I2C resources
  \return      \ref execution_status
*/
static int32_t I2C_MasterReceive(uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending, I2C_RESOURCES *const i2c) 
{

  uint8_t SRno;

  if ((data == NULL) || (num == 0))
  { 
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((i2c->info->flags & I2C_FLAG_POWERED) == 0)
  {
    // I2C is not powered 
    return ARM_DRIVER_ERROR;
  }

  // Check if receiver is busy
  if (i2c->info->status.busy || (XMC_GPIO_GetInput(i2c->sda_rx_port.port, i2c->sda_rx_port.pin) == 0))
  {
    return ARM_DRIVER_ERROR_BUSY;
  }

  SRno = (((uint32_t)(i2c->i2c) & 0x200U) != 0) ? 1 : 0;
#if (UC_SERIES == XMC14)
  SRno += (((uint32_t)(i2c->i2c) & 0x4000U) != 0) ? 1 : 0;
#endif

  XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_ERROR |
                                       XMC_I2C_CH_STATUS_FLAG_STOP_CONDITION_RECEIVED |
                                       XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED |
                                       XMC_I2C_CH_STATUS_FLAG_ARBITRATION_LOST |
                                       XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED);

  /* Set control variables */
  i2c->info->sla  = addr;
  i2c->info->xfer.rx_buf = data;
  i2c->info->xfer.rx_num  = num;
  i2c->info->xfer.rx_cnt  = 0;
  i2c->info->xfer.tx_num  = num;
  i2c->info->xfer.tx_cnt  = 0;
  
  /* Update driver status */
  i2c->info->status.busy             = 1;
  i2c->info->status.mode             = I2C_MODE_MASTER;
  i2c->info->status.direction        = I2C_DIR_READ;
  i2c->info->status.arbitration_lost = 0;
  i2c->info->status.bus_error        = 0;

  if(i2c->rx_fifo_size_reg == NO_FIFO)
  {
    XMC_USIC_CH_EnableEvent(i2c->i2c, XMC_USIC_CH_EVENT_STANDARD_RECEIVE | 
                                      XMC_USIC_CH_EVENT_ALTERNATIVE_RECEIVE);
    XMC_USIC_CH_SetInterruptNodePointer(i2c->i2c,XMC_USIC_CH_INTERRUPT_NODE_POINTER_RECEIVE, SRno);
    XMC_USIC_CH_SetInterruptNodePointer(i2c->i2c,XMC_USIC_CH_INTERRUPT_NODE_POINTER_ALTERNATE_RECEIVE, SRno);
  }
  else
  {
    XMC_USIC_CH_TXFIFO_Flush(i2c->i2c);   
    XMC_USIC_CH_RXFIFO_Flush(i2c->i2c);     
    XMC_USIC_CH_RXFIFO_EnableEvent(i2c->i2c, XMC_USIC_CH_RXFIFO_EVENT_CONF_STANDARD |
                                             XMC_USIC_CH_RXFIFO_EVENT_CONF_ALTERNATE); 
    XMC_USIC_CH_RXFIFO_SetInterruptNodePointer(i2c->i2c,XMC_USIC_CH_RXFIFO_INTERRUPT_NODE_POINTER_STANDARD, SRno);
    XMC_USIC_CH_RXFIFO_SetInterruptNodePointer(i2c->i2c,XMC_USIC_CH_RXFIFO_INTERRUPT_NODE_POINTER_ALTERNATE, SRno);
  
    if(num <= i2c->rx_fifo_size_num)
    {
      XMC_USIC_CH_RXFIFO_Configure(i2c->i2c,i2c->info->rx_fifo_pointer, (XMC_USIC_CH_FIFO_SIZE_t)i2c->rx_fifo_size_reg, i2c->info->xfer.rx_num - 1U); 
    }
    else
    {
      XMC_USIC_CH_RXFIFO_Configure(i2c->i2c,i2c->info->rx_fifo_pointer, (XMC_USIC_CH_FIFO_SIZE_t)i2c->rx_fifo_size_reg, i2c->rx_fifo_size_num - 1U); 
    }
  } 
  
  XMC_I2C_CH_EnableEvent(i2c->i2c, XMC_I2C_CH_EVENT_ACK |
                                   XMC_I2C_CH_EVENT_STOP_CONDITION_RECEIVED |
                                   XMC_I2C_CH_EVENT_NACK |
                                   XMC_I2C_CH_EVENT_ARBITRATION_LOST | 
                                   XMC_I2C_CH_EVENT_ERROR);                     

  XMC_I2C_CH_SetInterruptNodePointer(i2c->i2c, SRno);
  
  if(i2c->info->pending == false)
  {  
    /* send slave address and command to write */
    XMC_I2C_CH_MasterStart(i2c->i2c, addr, XMC_I2C_CH_CMD_READ);
  }
  else
  {
    XMC_I2C_CH_MasterRepeatedStart(i2c->i2c, addr, XMC_I2C_CH_CMD_READ);
  }
  
  i2c->info->pending = xfer_pending;

  return ARM_DRIVER_OK;
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_MasterReceive(uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending)
{
  return I2C_MasterReceive(addr, data, num, xfer_pending, i2c_resources[0]);
}
#endif

#if (RTE_I2C1 != 0)
static int32_t I2C1_MasterReceive(uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending)
{
  return I2C_MasterReceive(addr, data, num, xfer_pending, i2c_resources[1]);
}
#endif

#if (RTE_I2C2 != 0)
static int32_t I2C2_MasterReceive(uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending)
{
  return I2C_MasterReceive(addr, data, num, xfer_pending, i2c_resources[2]);
}
#endif

#if (RTE_I2C3 != 0)
static int32_t I2C3_MasterReceive(uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending)
{
  return I2C_MasterReceive(addr, data, num, xfer_pending, i2c_resources[3]);
}
#endif

#if (RTE_I2C4 != 0)
static int32_t I2C4_MasterReceive(uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending)
{
  return I2C_MasterReceive(addr, data, num, xfer_pending, i2c_resources[4]);
}
#endif

#if (RTE_I2C5 != 0)
static int32_t I2C5_MasterReceive(uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending)
{
  return I2C_MasterReceive(addr, data, num, xfer_pending, i2c_resources[5]);
}
#endif

  
/**
  \fn          int32_t I2C_SlaveTransmit (const uint8_t *data,
                                            uint32_t       num,
                                            I2C_RESOURCES *i2c)
  \brief       Start transmitting data as I2C Slave.
  \param[in]   data          Pointer to buffer with data to transmit to I2C Slave
  \param[in]   num           Number of data bytes to transmit
  \param[in]   i2c           Pointer to I2C resources
  \return      \ref execution_status
*/
static int32_t I2C_SlaveTransmit(const uint8_t *data, uint32_t num, I2C_RESOURCES *const i2c) 
{

  uint8_t SRno = 0U;
  
  if ((data == NULL) || (num == 0)) 
  {
    // Invalid parameters
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((i2c->info->flags & I2C_FLAG_POWERED) == 0)
  {
    // I2C is not powered 
    return ARM_DRIVER_ERROR;
  }

  if (i2c->info->status.busy)
  {
    /* Transfer operation in progress, or Slave stalled */
    return ARM_DRIVER_ERROR_BUSY;
  }

  SRno = (((uint32_t)(i2c->i2c) & 0x200U) != 0) ? 1 : 0;
#if (UC_SERIES == XMC14)
  SRno += (((uint32_t)(i2c->i2c) & 0x4000U) != 0) ? 1 : 0;
#endif

  XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_ERROR |
                                       XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED |
                                       XMC_I2C_CH_STATUS_FLAG_SLAVE_READ_REQUESTED |
                                       XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED);
  
  /* Set control variables */
  i2c->info->xfer.tx_buf    = (uint8_t *)data;
  i2c->info->xfer.tx_num    = num;
  i2c->info->xfer.tx_cnt    = 0;

  /* Update driver status */
  i2c->info->status.busy             = 1;
  i2c->info->status.mode             = I2C_MODE_SLAVE;
  i2c->info->status.direction        = I2C_DIR_WRITE;
  i2c->info->status.arbitration_lost = 0;
  i2c->info->status.bus_error        = 0;
 
  XMC_I2C_CH_EnableEvent(i2c->i2c, XMC_I2C_CH_EVENT_SLAVE_READ_REQUEST | 
                                   XMC_I2C_CH_EVENT_NACK | 
                                   XMC_I2C_CH_EVENT_ERROR);
 
  XMC_I2C_CH_SetInterruptNodePointer(i2c->i2c, SRno);
 
  if(i2c->tx_fifo_size_reg == NO_FIFO)
  {
    XMC_USIC_CH_EnableEvent(i2c->i2c, XMC_USIC_CH_EVENT_TRANSMIT_BUFFER) ;
    XMC_USIC_CH_SetInterruptNodePointer(i2c->i2c, XMC_USIC_CH_INTERRUPT_NODE_POINTER_TRANSMIT_BUFFER, SRno);
  }
  else
  {
    XMC_USIC_CH_TXFIFO_Flush(i2c->i2c); 
    XMC_USIC_CH_TXFIFO_EnableEvent(i2c->i2c, XMC_USIC_CH_TXFIFO_EVENT_CONF_STANDARD); 
    XMC_USIC_CH_TXFIFO_SetInterruptNodePointer(i2c->i2c, XMC_USIC_CH_TXFIFO_INTERRUPT_NODE_POINTER_STANDARD, SRno); 
  }
  
  if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_SLAVE_TRANSMIT);
  
  return ARM_DRIVER_OK;
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_SlaveTransmit(const uint8_t *data, uint32_t num) 
{
  return I2C_SlaveTransmit(data, num, i2c_resources[0]);
}
#endif

#if (RTE_I2C1 != 0)
static int32_t I2C1_SlaveTransmit(const uint8_t *data, uint32_t num) 
{
  return I2C_SlaveTransmit(data, num, i2c_resources[1]);
}
#endif

#if (RTE_I2C2 != 0)
static int32_t I2C2_SlaveTransmit(const uint8_t *data, uint32_t num) 
{
  return I2C_SlaveTransmit(data, num, i2c_resources[2]);
}
#endif

#if (RTE_I2C3 != 0)
static int32_t I2C3_SlaveTransmit(const uint8_t *data, uint32_t num) 
{
  return I2C_SlaveTransmit(data, num, i2c_resources[3]);
}
#endif

#if (RTE_I2C4 != 0)
static int32_t I2C4_SlaveTransmit(const uint8_t *data, uint32_t num) 
{
  return I2C_SlaveTransmit(data, num, i2c_resources[4]);
}
#endif

#if (RTE_I2C5 != 0)
static int32_t I2C5_SlaveTransmit(const uint8_t *data, uint32_t num) 
{
  return I2C_SlaveTransmit(data, num, i2c_resources[5]);
}
#endif  
  
  
/**
  \fn          int32_t I2C_SlaveReceive (uint8_t       *data,
                                          uint32_t       num,
                                          I2C_RESOURCES *i2c)
  \brief       Start receiving data as I2C Slave.
  \param[out]  data  Pointer to buffer for data to receive from I2C Master
  \param[in]   num   Number of data bytes to receive
  \param[in]   i2c   Pointer to I2C resources
  \return      \ref execution_status
*/
static int32_t I2C_SlaveReceive(uint8_t *data, uint32_t num, I2C_RESOURCES *const i2c) 
{

  uint8_t SRno;

  if ((data == NULL) || (num == 0))
  { 
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((i2c->info->flags & I2C_FLAG_POWERED) == 0)
  {
    // I2C is not powered 
    return ARM_DRIVER_ERROR;
  }

  // Check if receiver is busy
  if (i2c->info->status.busy == 1)
  {
    return ARM_DRIVER_ERROR_BUSY;
  }

  SRno = (((uint32_t)(i2c->i2c) & 0x200U) != 0) ? 1 : 1;
#if (UC_SERIES == XMC14)
  SRno += (((uint32_t)(i2c->i2c) & 0x4000U) != 0) ? 1 : 0;
#endif

  XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_ERROR |
                                       XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED |
                                       XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED);
  
  XMC_I2C_CH_EnableEvent(i2c->i2c, XMC_I2C_CH_EVENT_NACK | 
                                   XMC_I2C_CH_EVENT_ERROR);
 
  /* Set control variables */
  i2c->info->xfer.rx_buf    = data;
  i2c->info->xfer.rx_num     = num;
  i2c->info->xfer.rx_cnt     = 0;

  /* Update driver status */
  i2c->info->status.busy             = 1;
  i2c->info->status.mode             = I2C_MODE_SLAVE;
  i2c->info->status.direction        = I2C_DIR_READ;
  i2c->info->status.arbitration_lost = 0;
  i2c->info->status.bus_error        = 0;
  
  if(i2c->rx_fifo_size_reg == NO_FIFO)
  {
    XMC_USIC_CH_EnableEvent(i2c->i2c, XMC_USIC_CH_EVENT_STANDARD_RECEIVE | XMC_USIC_CH_EVENT_ALTERNATIVE_RECEIVE) ;
    XMC_USIC_CH_SetInterruptNodePointer(i2c->i2c, XMC_USIC_CH_INTERRUPT_NODE_POINTER_RECEIVE, SRno);
    XMC_USIC_CH_SetInterruptNodePointer(i2c->i2c, XMC_USIC_CH_INTERRUPT_NODE_POINTER_ALTERNATE_RECEIVE, SRno);
  }
  else
  {
    XMC_USIC_CH_RXFIFO_Flush(i2c->i2c);     
    XMC_USIC_CH_RXFIFO_EnableEvent(i2c->i2c, XMC_USIC_CH_RXFIFO_EVENT_CONF_STANDARD | 
                                             XMC_USIC_CH_RXFIFO_EVENT_CONF_ALTERNATE);
    XMC_USIC_CH_RXFIFO_SetInterruptNodePointer(i2c->i2c, XMC_USIC_CH_RXFIFO_INTERRUPT_NODE_POINTER_STANDARD, SRno);
    XMC_USIC_CH_RXFIFO_SetInterruptNodePointer(i2c->i2c, XMC_USIC_CH_RXFIFO_INTERRUPT_NODE_POINTER_ALTERNATE, SRno);
  
    if(num <= i2c->rx_fifo_size_num)
    {
      XMC_USIC_CH_RXFIFO_Configure(i2c->i2c, i2c->info->rx_fifo_pointer, (XMC_USIC_CH_FIFO_SIZE_t)i2c->rx_fifo_size_reg, num - 1U); 
    }
    else
    {
      XMC_USIC_CH_RXFIFO_Configure(i2c->i2c, i2c->info->rx_fifo_pointer, (XMC_USIC_CH_FIFO_SIZE_t)i2c->rx_fifo_size_reg, i2c->rx_fifo_size_num - 1U); 
    }   
  } 
  
  if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_SLAVE_RECEIVE); 
  
  return ARM_DRIVER_OK; 
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_SlaveReceive(uint8_t *data, uint32_t num) 
{
  return I2C_SlaveReceive(data, num, i2c_resources[0]);
}
#endif
#if (RTE_I2C1 != 0)
static int32_t I2C1_SlaveReceive(uint8_t *data, uint32_t num) 
{
  return I2C_SlaveReceive(data, num, i2c_resources[1]);
}
#endif
#if (RTE_I2C2 != 0)
static int32_t I2C2_SlaveReceive(uint8_t *data, uint32_t num) 
{
  return I2C_SlaveReceive(data, num, i2c_resources[2]);
}
#endif
#if (RTE_I2C3 != 0)
static int32_t I2C3_SlaveReceive(uint8_t *data, uint32_t num) 
{
  return I2C_SlaveReceive(data, num, i2c_resources[3]);
}
#endif
#if (RTE_I2C4 != 0)
static int32_t I2C4_SlaveReceive(uint8_t *data, uint32_t num) 
{
  return I2C_SlaveReceive(data, num, i2c_resources[4]);
}
#endif
#if (RTE_I2C5 != 0)
static int32_t I2C5_SlaveReceive(uint8_t *data, uint32_t num) 
{
  return I2C_SlaveReceive(data, num, i2c_resources[5]);
}
#endif

/**
  \fn          int32_t I2C_GetDataCount (I2C_RESOURCES *i2c)
  \brief       Get received data count.
  \param[in]   i2c     Pointer to I2C resources
  \return      number of data items 
*/
__STATIC_INLINE int32_t I2C_GetDataCount(I2C_RESOURCES *const i2c)
{
  if (i2c->info->status.direction == I2C_DIR_READ)
  {
    return i2c->info->xfer.rx_cnt;
  }
  else
  {
	return i2c->info->xfer.tx_cnt;
  }
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_GetDataCount(void) 
{
  return I2C_GetDataCount(i2c_resources[0]);
}
#endif
#if (RTE_I2C1 != 0)
static int32_t I2C1_GetDataCount(void) 
{
  return I2C_GetDataCount(i2c_resources[1]);
}
#endif
#if (RTE_I2C2 != 0)
static int32_t I2C2_GetDataCount(void) 
{
  return I2C_GetDataCount(i2c_resources[2]);
}
#endif
#if (RTE_I2C3 != 0)
static int32_t I2C3_GetDataCount(void) 
{
  return I2C_GetDataCount(i2c_resources[3]);
}
#endif
#if (RTE_I2C4 != 0)
static int32_t I2C4_GetDataCount(void) 
{
  return I2C_GetDataCount(i2c_resources[4]);
}
#endif
#if (RTE_I2C5 != 0)
static int32_t I2C5_GetDataCount(void) 
{
  return I2C_GetDataCount(i2c_resources[5]);
}
#endif

/**
  \fn          ARM_I2C_STATUS I2C_GetStatus (I2C_RESOURCES *i2c)
  \brief       Get I2C status.
  \param[in]   i2c     Pointer to I2C resources
  \return      I2C status \ref ARM_I2C_STATUS
*/
__STATIC_INLINE ARM_I2C_STATUS I2C_GetStatus(I2C_RESOURCES *const i2c) 
{
  return i2c->info->status;
}

#if (RTE_I2C0 != 0)
static ARM_I2C_STATUS I2C0_GetStatus(void) 
{
  return I2C_GetStatus(i2c_resources[0]);
}
#endif
#if (RTE_I2C1 != 0)
static ARM_I2C_STATUS I2C1_GetStatus(void) 
{
  return I2C_GetStatus(i2c_resources[1]);
}
#endif
#if (RTE_I2C2 != 0)
static ARM_I2C_STATUS I2C2_GetStatus(void) 
{
  return I2C_GetStatus(i2c_resources[2]);
}
#endif
#if (RTE_I2C3 != 0)
static ARM_I2C_STATUS I2C3_GetStatus(void) 
{
  return I2C_GetStatus(i2c_resources[3]);
}
#endif
#if (RTE_I2C4 != 0)
static ARM_I2C_STATUS I2C4_GetStatus(void) 
{
  return I2C_GetStatus(i2c_resources[4]);
}
#endif
#if (RTE_I2C5 != 0)
static ARM_I2C_STATUS I2C5_GetStatus(void) 
{
  return I2C_GetStatus(i2c_resources[5]);
}
#endif
  

/**
  \fn          int32_t I2C_Control (uint32_t control, uint32_t arg, I2C_RESOURCES *i2c)
  \brief       Control I2C Interface.
  \param[in]   control  Operation
  \param[in]   arg      Argument of operation (optional)
  \param[in]   i2c    Pointer to I2C resources
  \return      common \ref execution_status and driver specific \ref i2c_execution_status
*/
static int32_t I2C_Control(uint32_t control, uint32_t arg, I2C_RESOURCES *const i2c) 
{
  uint32_t sda_pin_state;

  if ((i2c->info->flags & I2C_FLAG_POWERED) == 0U) 
  {
    return ARM_DRIVER_ERROR;
  }

  switch (control)
  {
    default:
      return ARM_DRIVER_ERROR_UNSUPPORTED;
  
    case ARM_I2C_OWN_ADDRESS:
      // Set Own Slave Address; arg = slave address 
      if (arg != 0) 
      {
        XMC_I2C_CH_SetSlaveAddress(i2c->i2c, arg);
      }
      break;

    case ARM_I2C_BUS_CLEAR:     
      // Loop back mode
#if(UC_FAMILY == XMC1)
      XMC_I2C_CH_SetInputSource(i2c->i2c, XMC_I2C_CH_INPUT_SDA1, USIC0_C0_DX3_DOUT0);
      XMC_I2C_CH_SetInputSource(i2c->i2c, XMC_I2C_CH_INPUT_SDA, USIC0_C0_DX0_DX3INS);
#endif
#if(UC_FAMILY == XMC4)
      XMC_I2C_CH_SetInputSource(i2c->i2c, XMC_I2C_CH_INPUT_SDA, USIC0_C0_DX0_DOUT0);
#endif

      // Clear the bus by sending nine clock pulses
      XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_TRANSMIT_SHIFT_INDICATION);
      XMC_I2C_CH_MasterStart(i2c->i2c, 0xff, XMC_I2C_CH_CMD_READ);
      while ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & XMC_I2C_CH_STATUS_FLAG_TRANSMIT_SHIFT_INDICATION) == 0U);

      // Not Acknowledge software recovery since operating in Loop Back Mode
      if (i2c->tx_fifo_size_reg != NO_FIFO)
      {
    	  XMC_USIC_CH_TXFIFO_Flush(i2c->i2c);
      }
      if (i2c->rx_fifo_size_reg != NO_FIFO)
      {
    	  XMC_USIC_CH_RXFIFO_Flush(i2c->i2c);
      }
      XMC_USIC_CH_SetTransmitBufferStatus(i2c->i2c, XMC_USIC_CH_TBUF_STATUS_SET_IDLE);
      XMC_I2C_CH_ClearStatusFlag(i2c->i2c, 0x1ffff);

      XMC_I2C_CH_SetInputSource(i2c->i2c, XMC_I2C_CH_INPUT_SDA, i2c->sda_pin_input);
      sda_pin_state = XMC_GPIO_GetInput(i2c->sda_rx_port.port, i2c->sda_rx_port.pin);

      if (i2c->info->cb_event != NULL) 
      {
        /* Control operation has finished */
        i2c->info->cb_event (ARM_I2C_EVENT_BUS_CLEAR);
      }

      return (sda_pin_state == 0) ? ARM_DRIVER_ERROR : ARM_DRIVER_OK;
       
    case ARM_I2C_ABORT_TRANSFER:
      // Aborts the data transfer between Master and Slave for Transmit or Receive
      while (XMC_I2C_CH_Stop(i2c->i2c) != XMC_I2C_CH_STATUS_OK);
      i2c->info->status.busy = 0; 
      XMC_I2C_CH_Start(i2c->i2c);
      break;

    case ARM_I2C_BUS_SPEED:
      // Set Bus Speed in bps; arg = value      
      switch (arg) 
      {
        case ARM_I2C_BUS_SPEED_STANDARD:
          while (XMC_I2C_CH_Stop(i2c->i2c) != XMC_I2C_CH_STATUS_OK);
          XMC_I2C_CH_SetBaudrateEx(i2c->i2c, 100000, true);
          XMC_I2C_CH_Start(i2c->i2c);
          break;
        case ARM_I2C_BUS_SPEED_FAST:
          while (XMC_I2C_CH_Stop(i2c->i2c) != XMC_I2C_CH_STATUS_OK);
          XMC_I2C_CH_SetBaudrateEx(i2c->i2c, 400000, true);
          XMC_I2C_CH_Start(i2c->i2c);
          break;
        default:
          return ARM_DRIVER_ERROR_UNSUPPORTED;
      }     
      break;  
  } 
  return ARM_DRIVER_OK;
}

#if (RTE_I2C0 != 0)
static int32_t I2C0_Control(uint32_t control, uint32_t arg) 
{
  return I2C_Control(control, arg, i2c_resources[0]);
}
#endif
#if (RTE_I2C1 != 0)
static int32_t I2C1_Control(uint32_t control, uint32_t arg) 
{
  return I2C_Control(control, arg, i2c_resources[1]);
}
#endif
#if (RTE_I2C2 != 0)
static int32_t I2C2_Control(uint32_t control, uint32_t arg) 
{
  return I2C_Control(control, arg, i2c_resources[2]);
}
#endif
#if (RTE_I2C3 != 0)
static int32_t I2C3_Control(uint32_t control, uint32_t arg) 
{
  return I2C_Control(control, arg, i2c_resources[3]);
}
#endif
#if (RTE_I2C4 != 0)
static int32_t I2C4_Control(uint32_t control, uint32_t arg) 
{
  return I2C_Control(control, arg, i2c_resources[4]);
}
#endif
#if (RTE_I2C5 != 0)
static int32_t I2C5_Control(uint32_t control, uint32_t arg) 
{
  return I2C_Control(control, arg, i2c_resources[5]);
}
#endif
  
/**
  \fn          void I2C_IRQHandler (I2C_RESOURCES  *i2c)
  \brief       I2C Interrupt handler.
  \param[in]   i2c  Pointer to I2C resources
*/
static void I2C_IRQHandler(I2C_RESOURCES *const i2c)
{
  if (i2c->info->status.mode == I2C_MODE_MASTER) // If MASTER (read transaction ACK received)
  {
    if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & (XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED | XMC_I2C_CH_STATUS_FLAG_ERROR)) != 0)
    {
      XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED | XMC_I2C_CH_STATUS_FLAG_ERROR);
      i2c->info->status.bus_error = 1;
      i2c->info->status.busy = 0;
      if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE | ARM_I2C_EVENT_ADDRESS_NACK | ARM_I2C_EVENT_TRANSFER_INCOMPLETE);
      if (i2c->tx_fifo_size_reg != NO_FIFO)
      {
    	  XMC_USIC_CH_TXFIFO_Flush(i2c->i2c);
      }
      if (i2c->rx_fifo_size_reg != NO_FIFO)
      {
    	  XMC_USIC_CH_RXFIFO_Flush(i2c->i2c);
      }
      XMC_USIC_CH_SetTransmitBufferStatus(i2c->i2c, XMC_USIC_CH_TBUF_STATUS_SET_IDLE);
    }

    else if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & XMC_I2C_CH_STATUS_FLAG_ARBITRATION_LOST) != 0)
    {
      XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_ARBITRATION_LOST);
      i2c->info->status.arbitration_lost = 1; 
      i2c->info->status.busy = 0;
      if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE | ARM_I2C_EVENT_ARBITRATION_LOST | ARM_I2C_EVENT_TRANSFER_INCOMPLETE);
    }

    else if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & XMC_I2C_CH_STATUS_FLAG_STOP_CONDITION_RECEIVED) != 0)
    {
      XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_STOP_CONDITION_RECEIVED);
      if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE);
      
      // Clear TX busy flag
      i2c->info->status.busy = 0;
    }

    // Transmitter (ACK event)
    else if (i2c->info->status.direction == I2C_DIR_WRITE)
    {
      if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED) != 0)
      {
        XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED);
        if (i2c->info->xfer.tx_cnt < i2c->info->xfer.tx_num) 
        {         
          if (i2c->tx_fifo_size_reg == NO_FIFO)
          {
            i2c->info->xfer.tx_cnt++;          
            XMC_I2C_CH_MasterTransmit(i2c->i2c, *(i2c->info->xfer.tx_buf++)); 
          }
          else
          {
            /* Write to FIFO till Fifo is full */
            while (XMC_USIC_CH_TXFIFO_IsFull(i2c->i2c) == false)
            { 
              if (i2c->info->xfer.tx_cnt < i2c->info->xfer.tx_num) 
              { 
                i2c->info->xfer.tx_cnt++;         
                XMC_USIC_CH_TXFIFO_PutData(i2c->i2c, *(i2c->info->xfer.tx_buf++));
              }
              else
              {         
                break;
              }
            }
          }
        }
    
        else /*if (i2c->info->xfer.tx_cnt == i2c->info->xfer.tx_num)*/
        { 
          /* Disable standard transmit and error event interrupt */
          XMC_I2C_CH_DisableEvent(i2c->i2c, XMC_I2C_CH_EVENT_ACK);
          NVIC_ClearPendingIRQ(i2c->irq_num);
      
          if(i2c->info->pending == false)
          {
            XMC_I2C_CH_MasterStop(i2c->i2c); 
          }
          else
		  {
            i2c->info->status.busy = 0;
            if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE);      
		  }
        }
      }
    }
    
    // Receiver (standard/alternate receive event)
    else
    {
      if(i2c->rx_fifo_size_reg == NO_FIFO)
      {
        if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & (XMC_I2C_CH_STATUS_FLAG_RECEIVE_INDICATION | XMC_I2C_CH_STATUS_FLAG_ALTERNATIVE_RECEIVE_INDICATION)) != 0)
        {
          XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_RECEIVE_INDICATION | XMC_I2C_CH_STATUS_FLAG_ALTERNATIVE_RECEIVE_INDICATION);

          /* Read the data from FIFO buffer */
          i2c->info->xfer.rx_cnt++;   
          *(i2c->info->xfer.rx_buf++) = XMC_I2C_CH_GetReceivedData(i2c->i2c);  
          if (i2c->info->xfer.rx_cnt < i2c->info->xfer.rx_num)
          {
            XMC_I2C_CH_MasterReceiveAck(i2c->i2c);
          }
          else
          {
            XMC_I2C_CH_MasterReceiveNack(i2c->i2c);

            if(i2c->info->pending == false)
            {
              XMC_I2C_CH_MasterStop(i2c->i2c); 
            }
            else
            {							
              i2c->info->status.busy = 0;
              if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE);          
            }       
        
            XMC_I2C_CH_DisableEvent(i2c->i2c, XMC_USIC_CH_EVENT_STANDARD_RECEIVE | XMC_USIC_CH_EVENT_ALTERNATIVE_RECEIVE);       
          }   
        }
      }
      else // Using FIFO
      {
        if ((XMC_USIC_CH_RXFIFO_GetEvent(i2c->i2c) & (XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE)) != 0)
        {
          XMC_USIC_CH_RXFIFO_ClearEvent(i2c->i2c, XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE);

          uint32_t rx_remaining = i2c->info->xfer.rx_num - i2c->info->xfer.rx_cnt;
          while ((XMC_USIC_CH_RXFIFO_IsEmpty(i2c->i2c) == false))
          { 
            if (rx_remaining > 0)
            { 
              /* Read the data from FIFO buffer */
              i2c->info->xfer.rx_cnt++;
              rx_remaining--;
              *(i2c->info->xfer.rx_buf++) = XMC_I2C_CH_GetReceivedData(i2c->i2c);
            }
            else
            {       
              break;
            } 
          }
          
          if (rx_remaining > 0)
          {
            if (rx_remaining < i2c->rx_fifo_size_num)
            {
              XMC_USIC_CH_RXFIFO_SetTriggerLimit(i2c->i2c, rx_remaining - 1U); 
            }
          }
          else
          {
            XMC_USIC_CH_RXFIFO_DisableEvent(i2c->i2c,XMC_USIC_CH_RXFIFO_EVENT_CONF_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_CONF_ALTERNATE);
            if(i2c->info->pending == false)
            {
              XMC_I2C_CH_MasterStop(i2c->i2c);
            }
            else
            {
              i2c->info->status.busy = 0;
              if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE);
            }
          }

          uint32_t tx_remaining = i2c->info->xfer.tx_num - i2c->info->xfer.tx_cnt;
          if (tx_remaining > 0)
          {           
            for (uint32_t i = 0; i < i2c->tx_fifo_size_num; ++i)
            {         
              if (tx_remaining == 1)
              {
                XMC_I2C_CH_MasterReceiveNack(i2c->i2c);
              }
              else
              {
                XMC_I2C_CH_MasterReceiveAck(i2c->i2c);
              } 
              i2c->info->xfer.tx_cnt++;   
              tx_remaining--;

              if (tx_remaining == 0)
              {       
                break;
              }
            }
          } 
        }
      } // (XMC_I2C_CH_GetStatusFlag(i2c->i2c) & (XMC_I2C_CH_STATUS_FLAG_RECEIVE_INDICATION | XMC_I2C_CH_STATUS_FLAG_ALTERNATIVE_RECEIVE_INDICATION)) != 0

      if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED) != 0)
      {
        XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED);

        uint32_t tx_remaining = i2c->info->xfer.tx_num - i2c->info->xfer.tx_cnt;
        if(tx_remaining > 0)
        {
          if(i2c->tx_fifo_size_reg==NO_FIFO)
          {
            if(tx_remaining == 1)
            {
              XMC_I2C_CH_MasterReceiveNack(i2c->i2c);
            }
            else
            {
              XMC_I2C_CH_MasterReceiveAck(i2c->i2c);
            } 
            i2c->info->xfer.tx_cnt++;         
          }
          else
          {
            for (uint32_t i = 0; i < i2c->tx_fifo_size_num; ++i)
            {         
              if(tx_remaining == 1)
              {
                XMC_I2C_CH_MasterReceiveNack(i2c->i2c);
              }
              else
              {
                XMC_I2C_CH_MasterReceiveAck(i2c->i2c);
              } 
              i2c->info->xfer.tx_cnt++;
              tx_remaining--;   
      
              if (tx_remaining == 0)
              {
                break;
              }
            }
          }     
        }
      }
    }
  }
  else // if SLAVE
  {
    if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED) != 0U)
    {
      i2c->info->status.busy=0;
    }
    
    if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & XMC_I2C_CH_STATUS_FLAG_ERROR) != 0U)
    {
      /* I2C Bus error */
      i2c->info->status.busy=0;
      i2c->info->status.bus_error=1;      
    }
   
    if(i2c->info->status.direction == I2C_DIR_READ) // receiver
    {
      if ((XMC_USIC_CH_RXFIFO_GetEvent(i2c->i2c) & (XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE)) != 0)
      {
        XMC_USIC_CH_RXFIFO_ClearEvent(i2c->i2c, XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE);
      
        while((XMC_USIC_CH_RXFIFO_IsEmpty(i2c->i2c) == false))
        { 
          /* Read the data from FIFO buffer */
          i2c->info->xfer.rx_buf[i2c->info->xfer.rx_cnt++]= XMC_I2C_CH_GetReceivedData(i2c->i2c);
          if(i2c->info->xfer.rx_cnt ==  i2c->info->xfer.rx_num)
          {   
            XMC_USIC_CH_RXFIFO_DisableEvent(i2c->i2c,XMC_USIC_CH_RXFIFO_EVENT_CONF_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_CONF_ALTERNATE);
            // Clear RX busy flag and set receive transfer complete event
            i2c->info->status.busy = 0;
            if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE);
            break;
          }
        }
      
        uint32_t rx_remaining = i2c->info->xfer.rx_num - i2c->info->xfer.rx_cnt;
        if (rx_remaining < i2c->rx_fifo_size_num)
        {
          XMC_USIC_CH_RXFIFO_SetTriggerLimit(i2c->i2c, rx_remaining - 1U);
        }
      }
    }
    else // transmitter
    {
      if ((XMC_I2C_CH_GetStatusFlag(i2c->i2c) & XMC_I2C_CH_STATUS_FLAG_SLAVE_READ_REQUESTED) != 0U)
      {
        XMC_I2C_CH_ClearStatusFlag(i2c->i2c, XMC_I2C_CH_STATUS_FLAG_SLAVE_READ_REQUESTED);

        if(i2c->info->xfer.tx_num < i2c->info->xfer.tx_cnt) 
        { 
          /* Write to FIFO till Fifo is full */
          while((XMC_USIC_CH_TXFIFO_IsFull(i2c->i2c) == false))
          {   
            if(i2c->info->xfer.tx_num > i2c->info->xfer.tx_cnt) 
            { 
              XMC_I2C_CH_SlaveTransmit(i2c->i2c, i2c->info->xfer.tx_buf[i2c->info->xfer.tx_cnt]);
              i2c->info->xfer.tx_cnt++;
            }
            else
            {
              break;
            }
          }        
        }
        else  
        {     
          /* Disable standard transmit and error event interrupt */
          XMC_USIC_CH_TXFIFO_DisableEvent(i2c->i2c,XMC_USIC_CH_TXFIFO_EVENT_CONF_STANDARD); 
          // Clear TX busy flag
          i2c->info->status.busy=0; 
          if (i2c->info->cb_event) i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE);  
        }        
      }
    }
  }
}


#if (RTE_I2C0 != 0)
void I2C0_ISR(void) 
{
  I2C_IRQHandler(i2c_resources[0]);
}
#endif

#if (RTE_I2C1 != 0)
void I2C1_ISR(void) 
{
  I2C_IRQHandler(i2c_resources[1]);
}
#endif

#if (RTE_I2C2 != 0)
void I2C2_ISR(void) 
{
  I2C_IRQHandler(i2c_resources[2]);
}
#endif

#if (RTE_I2C3 != 0)
void I2C3_ISR(void) 
{
  I2C_IRQHandler(i2c_resources[3]);
}
#endif

#if (RTE_I2C4 != 0)
void I2C4_ISR(void) 
{
  I2C_IRQHandler(i2c_resources[4]);
}
#endif

#if (RTE_I2C5 != 0)
void I2C5_ISR(void) 
{
  I2C_IRQHandler(i2c_resources[5]);
}
#endif

#if (RTE_I2C0 != 0)
ARM_DRIVER_I2C Driver_I2C0 = 
{
  I2CX_GetVersion,
  I2C0_GetCapabilities,
  I2C0_Initialize,
  I2C0_Uninitialize,
  I2C0_PowerControl,
  I2C0_MasterTransmit,
  I2C0_MasterReceive, 
  I2C0_SlaveTransmit,
  I2C0_SlaveReceive,
  I2C0_GetDataCount,
  I2C0_Control,
  I2C0_GetStatus
};
#endif

#if (RTE_I2C1 != 0)
ARM_DRIVER_I2C Driver_I2C1 = 
{
  I2CX_GetVersion,
  I2C1_GetCapabilities,
  I2C1_Initialize,
  I2C1_Uninitialize,
  I2C1_PowerControl,
  I2C1_MasterTransmit,
  I2C1_MasterReceive, 
  I2C1_SlaveTransmit,
  I2C1_SlaveReceive,
  I2C1_GetDataCount,
  I2C1_Control,
  I2C1_GetStatus
};
#endif

#if (RTE_I2C2 != 0)
ARM_DRIVER_I2C Driver_I2C2 = 
{
  I2CX_GetVersion,
  I2C2_GetCapabilities,
  I2C2_Initialize,
  I2C2_Uninitialize,
  I2C2_PowerControl,
  I2C2_MasterTransmit,
  I2C2_MasterReceive, 
  I2C2_SlaveTransmit,
  I2C2_SlaveReceive,
  I2C2_GetDataCount,
  I2C2_Control,
  I2C2_GetStatus
};
#endif

#if (RTE_I2C3 != 0)
ARM_DRIVER_I2C Driver_I2C3 = 
{
  I2CX_GetVersion,
  I2C3_GetCapabilities,
  I2C3_Initialize,
  I2C3_Uninitialize,
  I2C3_PowerControl,
  I2C3_MasterTransmit,
  I2C3_MasterReceive, 
  I2C3_SlaveTransmit,
  I2C3_SlaveReceive,
  I2C3_GetDataCount,
  I2C3_Control,
  I2C3_GetStatus
};
#endif

#if (RTE_I2C4 != 0)
ARM_DRIVER_I2C Driver_I2C4 = 
{
  I2CX_GetVersion,
  I2C4_GetCapabilities,
  I2C4_Initialize,
  I2C4_Uninitialize,
  I2C4_PowerControl,
  I2C4_MasterTransmit,
  I2C4_MasterReceive, 
  I2C4_SlaveTransmit,
  I2C4_SlaveReceive,
  I2C4_GetDataCount,
  I2C4_Control,
  I2C4_GetStatus
};
#endif

#if (RTE_I2C5 != 0)
ARM_DRIVER_I2C Driver_I2C5 = 
{
  I2CX_GetVersion,
  I2C5_GetCapabilities,
  I2C5_Initialize,
  I2C5_Uninitialize,
  I2C5_PowerControl,
  I2C5_MasterTransmit,
  I2C5_MasterReceive, 
  I2C5_SlaveTransmit,
  I2C5_SlaveReceive,
  I2C5_GetDataCount,
  I2C5_Control,
  I2C5_GetStatus
};
#endif

#endif
