/*
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.                        
 *                                             
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *                                                                              
 * To improve the quality of the software, users are encouraged to share 
 * modifications, enhancements or bug fixes with Infineon Technologies AG 
 * at XMCSupport@infineon.com.
 *
 */

/**
 * @file SPI.h
 * @date 19 Nov, 2020
 * @version 2.7
 *
 * @brief SPI Driver for Infineon XMC devices
 *
 * History
 *
 * Version 1.0 Initial version<br>
 * Version 2.4 Added Status Structure for handling the driver status<br>
 * Version 2.5 Adapted to xmc1400 devices<br>
 * Version 2.6 Fixed SPI_INFO.mode variable type<br>
 * Version 2.7 Fixed compiler warnings
 */

#include "xmc_scu.h"
#include "xmc_gpio.h"
#include "xmc_spi.h"

#include "Driver_SPI.h"
#include "RTE_Components.h"
#include "RTE_Device.h"

// SPI flags
#define SPI_INITIALIZED       (1 << 0)
#define SPI_POWERED           (1 << 1)

#if (UC_SERIES == XMC14)
void IRQ9_Handler(void);
void IRQ10_Handler(void);
void IRQ11_Handler(void);
void IRQ12_Handler(void);

/*!< USIC SR0 Interrupt */
#define USIC0_0_IRQn 9U
#define SPI0_ISR IRQ9_Handler

/*!< USIC SR2 Interrupt (TX) */
#define USIC0_1_IRQn 10U
#define SPI1_ISR IRQ10_Handler

/*!< USIC SR0 Interrupt (TX) */
#define USIC1_0_IRQn 11U
#define SPI2_ISR IRQ11_Handler

/*!< USIC SR2 Interrupt (TX) */
#define USIC1_1_IRQn 12U
#define SPI3_ISR IRQ12_Handler

#else

void USIC0_0_IRQHandler(void);
void USIC0_1_IRQHandler(void);
void USIC1_0_IRQHandler(void);
void USIC1_1_IRQHandler(void);
void USIC2_0_IRQHandler(void);
void USIC2_1_IRQHandler(void);

#define SPI0_ISR USIC0_0_IRQHandler
#define SPI1_ISR USIC0_1_IRQHandler
#define SPI2_ISR USIC1_0_IRQHandler
#define SPI3_ISR USIC1_1_IRQHandler
#define SPI4_ISR USIC2_0_IRQHandler
#define SPI5_ISR USIC2_1_IRQHandler

#endif


// GPIO
typedef struct XMC_GPIO 
{
  XMC_GPIO_PORT_t *const port;
  const uint8_t pin;
} XMC_GPIO_t;

// SPI Transfer Information (Run-Time)
typedef struct SPI_TRANSFER_INFO 
{
  volatile uint32_t                num;            // Total number of data
  volatile uint8_t                *rx_buf;        // Pointer to in data buffer
  volatile uint8_t                *tx_buf;        // Pointer to out data buffer
  volatile uint32_t                rx_cnt;         // Number of data received
  volatile uint32_t                tx_cnt;         // Number of data transmitted
  uint8_t                          def_val;        // Default transfer value
  volatile uint32_t                baudrate;       // Baudrate value
  volatile uint8_t                 data_bytes;     // Number of data bytes
  volatile uint8_t                 bit_order;      // Bit order (MSB to LSB or LSB to MSB)
} SPI_TRANSFER_INFO_t ;

// SPI Information (Run-Time)
typedef struct 
{
  ARM_SPI_SignalEvent_t          cb_event;                  // Event callback
  ARM_SPI_STATUS                 status;                    // Status flags
  volatile SPI_TRANSFER_INFO_t   xfer;                      // Transfer information
  uint32_t                       mode;                      // SPI mode
  uint8_t                        flags;                     // SPI driver flags
  volatile uint32_t              rx_fifo_pointer;           // FIFO rx pointer
  volatile uint32_t              tx_fifo_pointer;           // FIFO tx pointer
} SPI_INFO;



// SPI Resources definitions
typedef struct 
{
  XMC_GPIO_t               miso_mosi_port;                            // MISO(master) MOSI(slave)  Pin identifier
  XMC_GPIO_CONFIG_t       *pin_miso_mosi_config;                      // MISO(master) MOSI(slave)  Pin configuration
  uint8_t                  miso_mosi_input;                           // MISO(master) MOSI(slave)  Input configuration
  XMC_GPIO_t               mosi_miso_port;                            // MOSI(master) MISO(slave) Pin identifier
  XMC_GPIO_CONFIG_t       *pin_mosi_miso_config;                      // MOSI(master) MISO(slave)  Pin configuration
  uint32_t                 pin_mosi_miso_alternate_function;          // MOSI(master) MISO(slave) pin alternate function
  XMC_GPIO_t               output_clck_port;                          // Output CLCK  Pin identifier
  XMC_GPIO_CONFIG_t       *pin_output_clck_config;                    // Output CLCK  Pin configuration
  uint32_t                 pin_output_clck_alternate_function;        // Output CLCK pin alternate function
  XMC_GPIO_t               input_clck_port;                           // Input CLCK  Pin identifier
  XMC_GPIO_CONFIG_t       *pin_input_clck_config;                     // Input CLCK  Pin configuration
  uint8_t                  clck_input;                                // Input CLCK  Input configuration
  XMC_GPIO_t               slave_select_output_port;                  // SELO  Pin identifier
  XMC_GPIO_CONFIG_t       *pin_slave_select_output_config;            // SELO  Pin configuration
  uint32_t                 pin_slave_select_output_alternate_function;// Output CLCK pin alternate function
  XMC_GPIO_t               slave_select_input_port;                   // SELI  Pin identifier
  XMC_GPIO_CONFIG_t       *pin_slave_select_input_config;             // SELI  Pin configuration
  uint8_t                  slave_select_input;                        // SELI  Input
  XMC_USIC_CH_t           *spi;                                       // Pointer to SPI peripheral
  IRQn_Type                irq_rx_num;                                // SPI RX IRQ Number
  uint32_t                 irq_priority;                              // SPI RX IRQ priority
  volatile uint32_t        tx_fifo_size_reg;                          // FIFO tx size register num
  volatile uint32_t        tx_fifo_size_num;                          // FIFO tx size register 
  volatile uint32_t        rx_fifo_size_reg;                          // FIFO rx size register num
  volatile uint32_t        rx_fifo_size_num;                          // FIFO rx size register
  volatile SPI_INFO       *info;                                      // Run-Time Information
  uint32_t                 spi_slave_select_line0;                 // Slave Select Line 0
  uint32_t                 spi_slave_select_line1;                 // Slave Select Line 1
  uint32_t                 spi_slave_select_line2;                 // Slave Select Line 2
  uint32_t                 spi_slave_select_line3;                 // Slave Select Line 3
  uint32_t                 spi_slave_select_line4;                 // Slave Select Line 4
  uint32_t                 spi_slave_select_line5;                 // Slave Select Line 5
  uint32_t                 spi_slave_select_line6;                 // Slave Select Line 6
  uint32_t                 spi_slave_select_line7;                 // Slave Select Line 7
} const SPI_RESOURCES;

#if (RTE_SPI0 != 0)
extern void SPI0_ISR(void);
extern ARM_DRIVER_SPI Driver_SPI0;
#endif

#if (RTE_SPI1 != 0)
extern void SPI1_ISR(void);
extern ARM_DRIVER_SPI Driver_SPI1;
#endif

#if (RTE_SPI2 != 0)
extern void SPI2_ISR(void);
extern ARM_DRIVER_SPI Driver_SPI2;
#endif

#if (RTE_SPI3 != 0)
extern void SPI3_ISR(void);
extern ARM_DRIVER_SPI Driver_SPI3;
#endif

#if (RTE_SPI4 != 0)
extern void SPI4_ISR(void);
extern ARM_DRIVER_SPI Driver_SPI4;
#endif

#if (RTE_SPI5 != 0)
extern void SPI5_ISR(void);
extern ARM_DRIVER_SPI Driver_SPI5;
#endif

