/*
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.                        
 *                                             
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *                                                                              
 * To improve the quality of the software, users are encouraged to share 
 * modifications, enhancements or bug fixes with Infineon Technologies AG 
 * at XMCSupport@infineon.com.
 *
 */

/**
 * @file UART.c
 * @date 19 Nov., 2020
 * @version 2.15
 *
 * @brief UART Driver for Infineon XMC devices
 *
 * History
 *
 * Version 1.0 Initial version<br>
 *
 * Version 2.4 Added fixed to prevent race conditions 
 *               and Initialize/Uninitialize, Power Control 
 *               guidelines related modifications <br>
 *
 * Version 2.5 priority for receive interrupt modified and fixed bug when using Tx FIFO <br>
 *
 * Version 2.6 Adapted to xmc1400 devices<br>
 *
 * Version 2.7 Fixed IRQ handlers names
 *
 * Version 2.8 Fixed IRQ handlers and IRQ numbers for XMC1400. Fixed FIFO pointers
 *
 * Version 2.9 Fixed UART_TX_ISR when using FIFO and length of data array to be sent is 1
 *
 * Version 2.10 Fix compiler portability
 *
 * Version 2.11 Fixed UART_INFO.mode variable type
 *
 * Version 2.12 Conditional compiling based on RTE_Drivers_USART
 *
 * Version 2.13 Fix compiler warnings
 *
 * Version 2.14 Added interrupt priority
 *              Fix Receive functionality if FIFO enabled
 *
 * Version 2.15 Fixed compiler warnings
 */

#include "UART.h"

#if defined(RTE_Drivers_USART)

#if (!RTE_UART0 && !RTE_UART1 && !RTE_UART2 && !RTE_UART3 && !RTE_UART4 && !RTE_UART5)
#error "USART not configured in RTE_Device.h!"
#endif

#define ARM_USART_DRV_VERSION ARM_DRIVER_VERSION_MAJOR_MINOR(2,15)   /* driver version */

// Driver Version
static const ARM_DRIVER_VERSION DriverVersion = 
{
  ARM_USART_API_VERSION,
  ARM_USART_DRV_VERSION
};


#if (defined(RTE_Drivers_USART) && \
     (RTE_UART0 == 0) && \
     (RTE_UART1 == 0) && \
     (RTE_UART2 == 0) && \
     (RTE_UART3 == 0) && \
     (RTE_UART4 == 0) && \
     (RTE_UART5 == 0))
#error "UART not configured in RTE_Device.h!"
#endif

// Default UART initialization
static const XMC_UART_CH_CONFIG_t uart_default_config =
{
  .baudrate = 100000U,
  .data_bits = 8U,
  .frame_length = 8U,
  .stop_bits = 1U,
  .parity_mode = XMC_USIC_CH_PARITY_MODE_NONE
};


static const ARM_USART_CAPABILITIES DriverCapabilities =
{
  .asynchronous = 1,      ///< supports UART (Asynchronous) mode
  .single_wire = 1,      ///< supports UART Single-wire mode
  .event_tx_complete = 1,      ///< Transmit completed event: \ref ARM_USART_EVENT_TX_COMPLETE
};

/* UART0 */
#if (RTE_UART0 != 0)

static volatile UART_INFO UART0_Info;
static XMC_GPIO_CONFIG_t UART0_rx_conf; 
static XMC_GPIO_CONFIG_t UART0_tx_conf; 

/* UART0 Resources */
static const UART_RESOURCES UART0_Resources = 
{
  {RTE_UART0_TX_PORT},
  &UART0_tx_conf,
  RTE_UART0_TX_AF,
  {RTE_UART0_RX_PORT},
  &UART0_rx_conf,
  RTE_UART0_RX_INPUT,
  XMC_UART0_CH0,
  (IRQn_Type)USIC0_0_IRQn,
  RTE_UART0_IRQ_PRIORITY,
  RTE_UART0_TX_FIFO_SIZE,
  RTE_UART0_TX_FIFO_SIZE_NUM,
  RTE_UART0_RX_FIFO_SIZE,
  RTE_UART0_RX_FIFO_SIZE_NUM,
  &UART0_Info
};
#endif

/* UART1 */
#if (RTE_UART1 != 0)

static volatile UART_INFO UART1_Info;
static XMC_GPIO_CONFIG_t UART1_rx_conf; 
static XMC_GPIO_CONFIG_t UART1_tx_conf; 

/* UART1 Resources */
static const UART_RESOURCES UART1_Resources = 
{
  {RTE_UART1_TX_PORT},
  &UART1_tx_conf,
  RTE_UART1_TX_AF,
  {RTE_UART1_RX_PORT},
  &UART1_rx_conf,
  RTE_UART1_RX_INPUT,
  XMC_UART0_CH1,
  (IRQn_Type)USIC0_1_IRQn,
  RTE_UART1_IRQ_PRIORITY,
  RTE_UART1_TX_FIFO_SIZE,
  RTE_UART1_TX_FIFO_SIZE_NUM,
  RTE_UART1_RX_FIFO_SIZE,
  RTE_UART1_RX_FIFO_SIZE_NUM,
  &UART1_Info
};
#endif

/* UART2 */
#if (RTE_UART2 != 0)

static volatile UART_INFO UART2_Info;
static XMC_GPIO_CONFIG_t UART2_rx_conf; 
static XMC_GPIO_CONFIG_t UART2_tx_conf; 

/* UART2 Resources */
static const UART_RESOURCES UART2_Resources = 
{
  {RTE_UART2_TX_PORT},
  &UART2_tx_conf,
  RTE_UART2_TX_AF,
  {RTE_UART2_RX_PORT},
  &UART2_rx_conf,
  RTE_UART2_RX_INPUT,
  XMC_UART1_CH0,
  (IRQn_Type)USIC1_0_IRQn,
  RTE_UART2_IRQ_PRIORITY,
  RTE_UART2_TX_FIFO_SIZE,
  RTE_UART2_TX_FIFO_SIZE_NUM,
  RTE_UART2_RX_FIFO_SIZE,
  RTE_UART2_RX_FIFO_SIZE_NUM,
  &UART2_Info
};
#endif


/* UART3 */
#if (RTE_UART3 != 0)

static volatile UART_INFO UART3_Info;
static XMC_GPIO_CONFIG_t UART3_rx_conf; 
static XMC_GPIO_CONFIG_t UART3_tx_conf; 

/* UART3 Resources */
static const UART_RESOURCES UART3_Resources = 
{
  {RTE_UART3_TX_PORT},
  &UART3_tx_conf,
  RTE_UART3_TX_AF,
  {RTE_UART3_RX_PORT},
  &UART3_rx_conf,
  RTE_UART3_RX_INPUT,
  XMC_UART1_CH1,
  (IRQn_Type)USIC1_1_IRQn,
  RTE_UART3_IRQ_PRIORITY,
  RTE_UART3_TX_FIFO_SIZE,
  RTE_UART3_TX_FIFO_SIZE_NUM,
  RTE_UART3_RX_FIFO_SIZE,
  RTE_UART3_RX_FIFO_SIZE_NUM,
  &UART3_Info
};
#endif


/* UART4 */
#if (RTE_UART4 != 0)

static volatile UART_INFO UART4_Info;
static XMC_GPIO_CONFIG_t UART4_rx_conf; 
static XMC_GPIO_CONFIG_t UART4_tx_conf; 

/* UART4 Resources */
static const UART_RESOURCES UART4_Resources = 
{
  {RTE_UART4_TX_PORT},
  &UART4_tx_conf,
  RTE_UART4_TX_AF,
  {RTE_UART4_RX_PORT},
  &UART4_rx_conf,
  RTE_UART4_RX_INPUT,
  XMC_UART2_CH0,
  (IRQn_Type)USIC2_0_IRQn,
  RTE_UART4_IRQ_PRIORITY,
  RTE_UART4_TX_FIFO_SIZE,
  RTE_UART4_TX_FIFO_SIZE_NUM,
  RTE_UART4_RX_FIFO_SIZE,
  RTE_UART4_RX_FIFO_SIZE_NUM,
  &UART4_Info
};
#endif


/* UART5 */
#if (RTE_UART5 != 0)

static volatile UART_INFO UART5_Info;
static XMC_GPIO_CONFIG_t UART5_rx_conf; 
static XMC_GPIO_CONFIG_t UART5_tx_conf; 

/* UART5 Resources */
static const UART_RESOURCES UART5_Resources = 
{
  {RTE_UART5_TX_PORT},
  &UART5_tx_conf,
  RTE_UART5_TX_AF,
  {RTE_UART5_RX_PORT},
  &UART5_rx_conf,
  RTE_UART5_RX_INPUT,
  XMC_UART2_CH1,
  (IRQn_Type)USIC2_1_IRQn,
  RTE_UART5_IRQ_PRIORITY,
  RTE_UART5_TX_FIFO_SIZE,
  RTE_UART5_TX_FIFO_SIZE_NUM,
  RTE_UART5_RX_FIFO_SIZE,
  RTE_UART5_RX_FIFO_SIZE_NUM,
  &UART5_Info
};

#endif


/* UART Resources */
static const UART_RESOURCES *const uart_resources[6] = 
{
#if (RTE_UART0 != 0)
  &UART0_Resources,
#else
  NULL,
#endif
#if (RTE_UART1 != 0)
  &UART1_Resources,
#else
  NULL,
#endif
#if (RTE_UART2 != 0)
  &UART2_Resources,
#else
  NULL,
#endif
#if (RTE_UART3 != 0)
  &UART3_Resources,
#else
  NULL,
#endif
#if (RTE_UART4 != 0)
  &UART4_Resources,
#else
  NULL,
#endif
#if (RTE_UART5 != 0)
  &UART5_Resources,
#else
  NULL,
#endif
};

static void UART_TX_ISR(UART_RESOURCES *const uart);
static void UART_RX_ISR(UART_RESOURCES *const uart);

/**
  \fn          ARM_DRV_VERSION USARTX_GetVersion (void)
  \brief       Get driver version.
  \return      \ref ARM_DRV_VERSION
*/
static ARM_DRIVER_VERSION USARTX_GetVersion(void)
{
  return DriverVersion;
}


/**
  \fn          ARM_UART_CAPABILITIES UART_GetCapabilities (UART_RESOURCES  *uart)
  \brief       Get driver capabilities.
  \param[in]   uart    Pointer to UART resources
  \return      \ref USART_CAPABILITIES
*/
__STATIC_INLINE ARM_USART_CAPABILITIES UART_GetCapabilities(UART_RESOURCES *const uart)
{
  (void)uart;
  return DriverCapabilities;
}

#if (RTE_UART0 != 0)
static ARM_USART_CAPABILITIES UART0_GetCapabilities(void)
{
  return UART_GetCapabilities(uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static ARM_USART_CAPABILITIES UART1_GetCapabilities(void)
{
  return UART_GetCapabilities(uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static ARM_USART_CAPABILITIES UART2_GetCapabilities(void)
{
  return UART_GetCapabilities(uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static ARM_USART_CAPABILITIES UART3_GetCapabilities(void)
{
  return UART_GetCapabilities(uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static ARM_USART_CAPABILITIES UART4_GetCapabilities(void)
{
  return UART_GetCapabilities(uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static ARM_USART_CAPABILITIES UART5_GetCapabilities(void)
{
  return UART_GetCapabilities(uart_resources[5]);
}
#endif

/**
  \fn          ARM_USART_STATUS UART_Initialize (ARM_USART_SignalEvent_t cb_event,
                                                 UART_RESOURCES *uart)
  \brief       Initialize UART Interface.
  \param[in]   cb_event Pointer to \ref ARM_USART_SignalEvent
  \param[in]   uart Pointer to USART resources
  \return      \ref ARM_USART_STATUS
*/
static int32_t UART_Initialize(ARM_USART_SignalEvent_t cb_event, UART_RESOURCES *const uart)
{
  if (((uart->info->flags) &UART_INITIALIZED) == 0)
  {
    // Initialize USART Run-Time Resources
    uart->info->cb_event= cb_event;

    uart->info->status.tx_busy          = 0;
    uart->info->status.rx_busy          = 0;
    uart->info->status.tx_underflow     = 0;
    uart->info->status.rx_overflow      = 0;
    uart->info->status.rx_break         = 0;
    uart->info->status.rx_framing_error = 0;
    uart->info->status.rx_parity_error  = 0;

    uart->info->flags |= UART_INITIALIZED;
  }
  return ARM_DRIVER_OK;
}

#if (RTE_UART0 != 0)
static int32_t UART0_Initialize(ARM_USART_SignalEvent_t cb_event) 
{
  return UART_Initialize(cb_event, uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static int32_t UART1_Initialize(ARM_USART_SignalEvent_t cb_event) 
{
  return UART_Initialize(cb_event, uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static int32_t UART2_Initialize(ARM_USART_SignalEvent_t cb_event) 
{
  return UART_Initialize(cb_event, uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static int32_t UART3_Initialize(ARM_USART_SignalEvent_t cb_event) 
{
  return UART_Initialize(cb_event, uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static int32_t UART4_Initialize(ARM_USART_SignalEvent_t cb_event) 
{
  return UART_Initialize(cb_event, uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static int32_t UART5_Initialize(ARM_USART_SignalEvent_t cb_event) 
{
  return UART_Initialize(cb_event, uart_resources[5]);
}
#endif


  /**
  \fn          int32_t UART_Uninitialize (UART_RESOURCES *uart)
  \brief       De-initialize USART Interface.
  \param[in]   uart  Pointer to USART resources
  \return      \ref execution_status
*/
__STATIC_INLINE int32_t UART_Uninitialize(UART_RESOURCES *const uart) 
{
  
  // Reset UART status flags
  uart->info->flags = 0;

  return ARM_DRIVER_OK;
}

#if (RTE_UART0 != 0)
static int32_t UART0_Uninitialize (void) 
{
  return UART_Uninitialize(uart_resources[0]);
}
#endif  

#if (RTE_UART1 != 0)
static int32_t UART1_Uninitialize (void) 
{
  return UART_Uninitialize(uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static int32_t UART2_Uninitialize (void) 
{
  return UART_Uninitialize(uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static int32_t UART3_Uninitialize (void) 
{
  return UART_Uninitialize(uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static int32_t UART4_Uninitialize (void) 
{
  return UART_Uninitialize(uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static int32_t UART5_Uninitialize (void) 
{
  return UART_Uninitialize(uart_resources[5]);
}
#endif

/**
  \fn          ARM_USART_STATUS UART_PowerControl (ARM_POWER_STATE  state,
                                                   UART_RESOURCES   *uart)
  \brief       Controls UART Interface Power.
  \param[in]   state    Power state
  \param[in]   uart Pointer to USART resources
  \return      \ref USART_STATUS
*/
static int32_t UART_PowerControl(ARM_POWER_STATE state, UART_RESOURCES *const uart)
{
  if (state == ARM_POWER_FULL) 
  {
    if ((uart->info->flags & UART_INITIALIZED) == 0U) 
    {
      return ARM_DRIVER_ERROR;
    }
    if ((uart->info->flags & UART_POWERED)!= 0U) 
    {
      return ARM_DRIVER_OK;
    }

    if (uart->tx_fifo_size_num > uart->rx_fifo_size_num)
    {
      uart->info->tx_fifo_pointer = (((uint32_t)(uart->uart) & 0x200U) != 0) ? 32 : 0;
      uart->info->rx_fifo_pointer = uart->info->tx_fifo_pointer + uart->tx_fifo_size_num;
    }
    else
    {
      uart->info->rx_fifo_pointer = (((uint32_t)(uart->uart) & 0x200U) != 0) ? 32 : 0;
      uart->info->tx_fifo_pointer = uart->info->rx_fifo_pointer + uart->rx_fifo_size_num;
    }
    
    XMC_UART_CH_Init(uart->uart, &uart_default_config);   
    XMC_USIC_CH_TXFIFO_Configure(uart->uart, uart->info->tx_fifo_pointer, (XMC_USIC_CH_FIFO_SIZE_t)uart->tx_fifo_size_reg, 1U); 
    XMC_UART_CH_SetInputSource(uart->uart, XMC_UART_CH_INPUT_RXD, uart->input);
#if(UC_FAMILY == XMC1)
    XMC_UART_CH_SetInputSource(uart->uart, XMC_UART_CH_INPUT_RXD1, 0);
#endif
    NVIC_ClearPendingIRQ(uart->irq_num);

#if(UC_FAMILY == XMC4)
    // Receive interrupt has higher priority as transmit interrupt
    NVIC_SetPriority(uart->irq_num, NVIC_EncodePriority(NVIC_GetPriorityGrouping(), uart->irq_priority, 0U));
#else
    // Receive interrupt has higher priority as transmit interrupt
    NVIC_SetPriority(uart->irq_num, uart->irq_priority);
#endif

#if (UC_SERIES == XMC14)    
    XMC_SCU_SetInterruptControl(uart->irq_num, (XMC_SCU_IRQCTRL_t)(((uart->irq_num) << 8U) | 3U));
#endif
         
    NVIC_EnableIRQ(uart->irq_num); 
   
    uart->info->flags |= UART_POWERED;
  }
  else if (state == ARM_POWER_OFF) 
  {
    NVIC_DisableIRQ(uart->irq_num); 

    // Disable events
    XMC_UART_CH_DisableEvent(uart->uart, XMC_USIC_CH_EVENT_TRANSMIT_BUFFER | XMC_USIC_CH_EVENT_STANDARD_RECEIVE | XMC_USIC_CH_EVENT_ALTERNATIVE_RECEIVE);
    XMC_USIC_CH_TXFIFO_DisableEvent(uart->uart, XMC_USIC_CH_TXFIFO_EVENT_CONF_STANDARD);
    XMC_USIC_CH_RXFIFO_DisableEvent(uart->uart, XMC_USIC_CH_RXFIFO_EVENT_CONF_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_CONF_ALTERNATE);
    
    // Disable peripheral, terminates any pending data transfers
    while (XMC_UART_CH_Stop(uart->uart) == XMC_UART_CH_STATUS_BUSY)
    {
      ;
    }
  
    // Invalidate receive buffers
    (void)XMC_UART_CH_GetReceivedData(uart->uart);
    (void)XMC_UART_CH_GetReceivedData(uart->uart);
    
    XMC_USIC_CH_TXFIFO_Configure(uart->uart, 0, XMC_USIC_CH_FIFO_DISABLED, 0U); 
    XMC_USIC_CH_RXFIFO_Configure(uart->uart, 0, XMC_USIC_CH_FIFO_DISABLED, 0U); 
    
    uart->info->flags &= ~UART_POWERED;
  } 
  else
  {
    return ARM_DRIVER_ERROR_UNSUPPORTED;
  }

  return ARM_DRIVER_OK;
}

#if (RTE_UART0 != 0)
static int32_t UART0_PowerControl(ARM_POWER_STATE state)
{
  return UART_PowerControl(state, uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static int32_t UART1_PowerControl(ARM_POWER_STATE state)
{
  return UART_PowerControl(state, uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static int32_t UART2_PowerControl(ARM_POWER_STATE state)
{
  return UART_PowerControl(state, uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static int32_t UART3_PowerControl(ARM_POWER_STATE state)
{
  return UART_PowerControl(state, uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static int32_t UART4_PowerControl(ARM_POWER_STATE state)
{
  return UART_PowerControl(state, uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static int32_t UART5_PowerControl(ARM_POWER_STATE state)
{
  return UART_PowerControl(state, uart_resources[5]);
}
#endif

/**
  \fn          int32_t UART_Send (const uint8_t   *data,
                                  uint32_t num,
                                  UART_RESOURCES   *uart)
  \brief       Write data to UART transmitter.
  \param[in]   data  Pointer to buffer with data to write to UART transmitter
  \param[in]   num  Data buffer size in bytes
  \param[in]   uart Pointer to UART resources
  \return      driver status
*/
static int32_t UART_Send(const void *data, uint32_t num, UART_RESOURCES *const uart)
{
  uint8_t SRno;
  
  if ((data == NULL) || (num == 0)) 
  {
    // Invalid parameters
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((uart->info->flags & UART_POWERED) == 0)
  {
    // UART is not powered 
    return ARM_DRIVER_ERROR;
  }

  if (uart->info->status.tx_busy == 1)
  {
    // Send is not completed yet
    return ARM_DRIVER_ERROR_BUSY;
  }

  uart->info->status.tx_busy = 1;
  
  // Save transmit buffer info
  uart->info->xfer.tx_buf = (uint8_t *)data;
  uart->info->xfer.tx_num = num;
  uart->info->xfer.tx_cnt = 0;

  SRno = (((uint32_t)(uart->uart) & 0x200U) != 0) ? 1 : 0;
#if (UC_SERIES == XMC14)
  SRno += (((uint32_t)(uart->uart) & 0x4000U) != 0) ? 1 : 0;
#endif

  XMC_USIC_CH_TXFIFO_Flush(uart->uart); 
  UART_TX_ISR(uart);

  if(uart->tx_fifo_size_reg==NO_FIFO)
  {
    XMC_USIC_CH_EnableEvent(uart->uart,XMC_USIC_CH_EVENT_TRANSMIT_BUFFER);
    XMC_USIC_CH_SetInterruptNodePointer(uart->uart, XMC_USIC_CH_INTERRUPT_NODE_POINTER_TRANSMIT_BUFFER, SRno);
  }
  else
  {
    XMC_USIC_CH_TXFIFO_EnableEvent(uart->uart, XMC_USIC_CH_TXFIFO_EVENT_CONF_STANDARD); 
    XMC_USIC_CH_TXFIFO_SetInterruptNodePointer(uart->uart, XMC_USIC_CH_TXFIFO_INTERRUPT_NODE_POINTER_STANDARD, SRno);
  }
  

  return ARM_DRIVER_OK;
}

#if (RTE_UART0 != 0)
static int32_t UART0_Send(const void *data, uint32_t num)
{
  return UART_Send(data, num, uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static int32_t UART1_Send(const void *data, uint32_t num)
{
  return UART_Send(data, num, uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static int32_t UART2_Send(const void *data, uint32_t num)
{
  return UART_Send(data, num, uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static int32_t UART3_Send(const void *data, uint32_t num) 
{
  return UART_Send(data, num, uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static int32_t UART4_Send(const void *data, uint32_t num) 
{
  return UART_Send(data, num, uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static int32_t UART5_Send(const void *data, uint32_t num) 
{
  return UART_Send(data, num, uart_resources[5]);
}
#endif


/**
  \fn          int32_t UART_Receive (uint8_t *data, uint32_t num,
                                      UART_RESOURCES *uart)
  \brief       Read data from UART receiver.
  \param[out]  data  Pointer to buffer for data read from USART receiver
  \param[in]   size  Data buffer size in bytes
  \param[in]   uart Pointer to UART resources
  \return      driver status
*/
static int32_t UART_Receive(void *data, uint32_t num, UART_RESOURCES *const uart)
{
  uint8_t SRno;
 
  if ((data == NULL) || (num == 0))
  {
    // Invalid parameters
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((uart->info->flags & UART_POWERED) == 0)
  {
    // UART is not powered
    return ARM_DRIVER_ERROR;
  }

  // Check if receiver is busy
  if (uart->info->status.rx_busy == 1) 
  {
    return ARM_DRIVER_ERROR_BUSY;
  }
  
  // Set RX busy flag
  uart->info->status.rx_busy = 1;

  // Clear RX status
  uart->info->status.rx_break          = 0;
  uart->info->status.rx_framing_error  = 0;
  uart->info->status.rx_overflow       = 0;
  uart->info->status.rx_parity_error   = 0;

  // Save receive buffer info
  uart->info->xfer.rx_buf = (uint8_t *)data;
  uart->info->xfer.rx_cnt = 0;                            
  // Save number of data to be received
  uart->info->xfer.rx_num = num;

  SRno = (((uint32_t)(uart->uart) & 0x200U) != 0) ? 1 : 0;
#if (UC_SERIES == XMC14)
  SRno += (((uint32_t)(uart->uart) & 0x4000U) != 0) ? 1 : 0;
#endif
  
  if(uart->rx_fifo_size_reg == NO_FIFO)
  {
    XMC_USIC_CH_EnableEvent(uart->uart, XMC_USIC_CH_EVENT_STANDARD_RECEIVE | XMC_USIC_CH_EVENT_ALTERNATIVE_RECEIVE);
    XMC_USIC_CH_SetInterruptNodePointer(uart->uart, XMC_USIC_CH_INTERRUPT_NODE_POINTER_RECEIVE, SRno);
    XMC_USIC_CH_SetInterruptNodePointer(uart->uart, XMC_USIC_CH_INTERRUPT_NODE_POINTER_ALTERNATE_RECEIVE, SRno);
  }
  else
  {
    XMC_USIC_CH_RXFIFO_Flush(uart->uart);     
    XMC_USIC_CH_RXFIFO_EnableEvent(uart->uart, XMC_USIC_CH_RXFIFO_EVENT_CONF_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_CONF_ALTERNATE);
    XMC_USIC_CH_RXFIFO_SetInterruptNodePointer(uart->uart, XMC_USIC_CH_RXFIFO_INTERRUPT_NODE_POINTER_STANDARD, SRno);
    XMC_USIC_CH_RXFIFO_SetInterruptNodePointer(uart->uart, XMC_USIC_CH_RXFIFO_INTERRUPT_NODE_POINTER_ALTERNATE, SRno);
  
    if(num <= uart->rx_fifo_size_num)
    {
      XMC_USIC_CH_RXFIFO_Configure(uart->uart, uart->info->rx_fifo_pointer, (XMC_USIC_CH_FIFO_SIZE_t)uart->rx_fifo_size_reg, num - 1U); 
    }
    else
    {
      XMC_USIC_CH_RXFIFO_Configure(uart->uart, uart->info->rx_fifo_pointer, (XMC_USIC_CH_FIFO_SIZE_t)uart->rx_fifo_size_reg, uart->rx_fifo_size_num - 1); 
    }
  }
  return ARM_DRIVER_OK;
}

#if (RTE_UART0 != 0)
static int32_t UART0_Receive(void *data, uint32_t num) 
{
  return UART_Receive(data, num, uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static int32_t UART1_Receive(void *data, uint32_t num) 
{
  return UART_Receive(data, num, uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static int32_t UART2_Receive(void *data, uint32_t num) 
{
  return UART_Receive(data, num, uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static int32_t UART3_Receive(void *data, uint32_t num) 
{
  return UART_Receive(data, num, uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static int32_t UART4_Receive(void *data, uint32_t num) 
{
  return UART_Receive(data, num, uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static int32_t UART5_Receive(void *data, uint32_t num) 
{
  return UART_Receive(data, num, uart_resources[5]);
}
#endif

/**
  \fn          int32_t UART_Transfer (const void *data_out,
                                             void *data_in,
                                             uint32_t num,
                                             UART_RESOURCES  *uart)
  \brief       Start sending/receiving data to/from USART transmitter/receiver.
  \param[in]   data_out  Pointer to buffer with data to send to UART transmitter
  \param[out]  data_in   Pointer to buffer for data to receive from UART receiver
  \param[in]   num       Number of data items to transfer
  \param[in]   uart     Pointer to UART resources
  \return      \ref execution_status
*/
__STATIC_INLINE int32_t UART_Transfer(const void  *data_out,
                                    void  *data_in,
                                    uint32_t  num,
                                    UART_RESOURCES *const uart) 
{
  (void)data_out;
  (void)data_in;
  (void)num;
  (void)uart;
  return ARM_DRIVER_OK;
}

#if (RTE_UART0 != 0)
static int32_t UART0_Transfer(const void *data_out,void  *data_in,uint32_t num) 
{
  return UART_Transfer(data_out, data_in, num, uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static int32_t UART1_Transfer(const void *data_out,void  *data_in,uint32_t num) 
{
  return UART_Transfer (data_out, data_in, num, uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static int32_t UART2_Transfer(const void *data_out,void  *data_in,uint32_t num) 
{
  return UART_Transfer (data_out, data_in, num, uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static int32_t UART3_Transfer(const void *data_out,void  *data_in,uint32_t num) 
{
  return UART_Transfer(data_out, data_in, num, uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static int32_t UART4_Transfer(const void *data_out,void  *data_in,uint32_t num) 
{
  return UART_Transfer(data_out, data_in, num, uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static int32_t UART5_Transfer(const void *data_out,void  *data_in,uint32_t num) 
{
  return UART_Transfer(data_out, data_in, num, uart_resources[5]);
}
#endif
  
/**
  \fn          uint32_t UART_GetTxCount (UART_RESOURCES *uart)
  \brief       Get transmitted data count.
  \param[in]   uart     Pointer to UART resources
  \return      number of data items transmitted
*/
__STATIC_INLINE uint32_t UART_GetTxCount(UART_RESOURCES *const uart) 
{
  return uart->info->xfer.tx_cnt;
}

#if (RTE_UART0 != 0)
static uint32_t UART0_GetTxCount(void) 
{
  return UART_GetTxCount(uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static uint32_t UART1_GetTxCount(void) 
{
  return UART_GetTxCount(uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static uint32_t UART2_GetTxCount(void) 
{
  return UART_GetTxCount(uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static uint32_t UART3_GetTxCount(void) 
{
  return UART_GetTxCount(uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static uint32_t UART4_GetTxCount(void) 
{
  return UART_GetTxCount(uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static uint32_t UART5_GetTxCount(void) 
{
  return UART_GetTxCount(uart_resources[5]);
}
#endif

/**
  \fn          uint32_t UART_GetRxCount (UART_RESOURCES *uart)
  \brief       Get received data count.
  \param[in]   uart     Pointer to UART resources
  \return      number of data items received
*/
__STATIC_INLINE uint32_t UART_GetRxCount(UART_RESOURCES *const uart)
{
  return uart->info->xfer.rx_cnt;
}

#if (RTE_UART0 != 0)
static uint32_t UART0_GetRxCount(void) 
{
  return UART_GetRxCount(uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static uint32_t UART1_GetRxCount(void) 
{
  return UART_GetRxCount(uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static uint32_t UART2_GetRxCount(void) 
{
  return UART_GetRxCount(uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static uint32_t UART3_GetRxCount(void) 
{
  return UART_GetRxCount(uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static uint32_t UART4_GetRxCount(void) 
{
  return UART_GetRxCount(uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static uint32_t UART5_GetRxCount(void) 
{
  return UART_GetRxCount(uart_resources[0]);
}
#endif

/**
  \fn          ARM_USART_STATUS UART_GetStatus (UART_RESOURCES *uart)
  \brief       Get UART status.
  \param[in]   uart     Pointer to USART resources
  \return      UART status \ref ARM_USART_STATUS
*/
__STATIC_INLINE ARM_USART_STATUS UART_GetStatus(UART_RESOURCES *const usart)
{
  return usart->info->status;
}

#if (RTE_UART0 != 0)
static ARM_USART_STATUS UART0_GetStatus(void) 
{
  return UART_GetStatus(uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static ARM_USART_STATUS UART1_GetStatus(void) 
{
  return UART_GetStatus(uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static ARM_USART_STATUS UART2_GetStatus(void) 
{
  return UART_GetStatus(uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static ARM_USART_STATUS UART3_GetStatus(void) 
{
  return UART_GetStatus(uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static ARM_USART_STATUS UART4_GetStatus(void) 
{
  return UART_GetStatus(uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static ARM_USART_STATUS UART5_GetStatus(void) 
{
  return UART_GetStatus(uart_resources[5]);
}
#endif

/**
  \fn          ARM_USART_STATUS UART_SetModemControl (ARM_USART_MODEM_CONTROL control,
                                                     UART_RESOURCES  *uart)
  \brief       Set USART Modem Control line state.
  \param[in]   control  \ref ARM_USART_MODEM_CONTROL
  \param[in]   uart Pointer to USART resources
  \return      \ref ARM_USART_STATUS
*/
__STATIC_INLINE int32_t UART_SetModemControl(ARM_USART_MODEM_CONTROL control,
                                           UART_RESOURCES *const uart) 
{
  (void)control;
  (void)uart;
  return ARM_DRIVER_ERROR;
}

#if (RTE_UART0 != 0)
static int32_t UART0_SetModemControl(ARM_USART_MODEM_CONTROL control) 
{
  return UART_SetModemControl(control, uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static int32_t UART1_SetModemControl(ARM_USART_MODEM_CONTROL control) 
{
  return UART_SetModemControl(control, uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static int32_t UART2_SetModemControl(ARM_USART_MODEM_CONTROL control) 
{
  return UART_SetModemControl(control, uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static int32_t UART3_SetModemControl(ARM_USART_MODEM_CONTROL control) 
{
  return UART_SetModemControl(control, uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static int32_t UART4_SetModemControl(ARM_USART_MODEM_CONTROL control) 
{
  return UART_SetModemControl(control, uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static int32_t UART5_SetModemControl(ARM_USART_MODEM_CONTROL control) 
{
  return UART_SetModemControl(control, uart_resources[5]);
}
#endif


/**
  \fn          ARM_USART_MODEM_STATUS UART_GetModemStatus (UART_RESOURCES  *uart)
  \brief       Get USART Modem Status lines state.
  \param[in]   uart Pointer to USART resources
  \return      \ref ARM_USART_MODEM_STATUS
*/
static ARM_USART_MODEM_STATUS UART_GetModemStatus(UART_RESOURCES *const uart)
{
  (void)uart;

  ARM_USART_MODEM_STATUS mst = {0};
  return mst;
}

#if (RTE_UART0 != 0)
static ARM_USART_MODEM_STATUS UART0_GetModemStatus(void) 
{
  return UART_GetModemStatus(uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static ARM_USART_MODEM_STATUS UART1_GetModemStatus(void) 
{
  return UART_GetModemStatus(uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static ARM_USART_MODEM_STATUS UART2_GetModemStatus(void) 
{
  return UART_GetModemStatus(uart_resources[2]);
}
#endif

#if (RTE_UART3 != 0)
static ARM_USART_MODEM_STATUS UART3_GetModemStatus(void) 
{
  return UART_GetModemStatus(uart_resources[3]);
}
#endif

#if (RTE_UART4 != 0)
static ARM_USART_MODEM_STATUS UART4_GetModemStatus(void) 
{
  return UART_GetModemStatus(uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static ARM_USART_MODEM_STATUS UART5_GetModemStatus(void) 
{
  return UART_GetModemStatus(uart_resources[5]);
}
#endif

/**
  \fn          int32_t UART_Control (uint32_t control,
                                      uint32_t arg,
                                      UART_RESOURCES  *uart)
  \brief       Control UART Interface.
  \param[in]   control  Operation
  \param[in]   arg      Argument of operation (optional)
  \param[in]   uart    Pointer to USART resources
  \return      common \ref execution_status and driver specific \ref usart_execution_status
*/
static int32_t UART_Control(uint32_t control, uint32_t arg, UART_RESOURCES *const uart) 
{
  if ((uart->info->flags & UART_POWERED) == 0U) 
  {
    return ARM_DRIVER_ERROR;
  }

  switch (control & ARM_USART_CONTROL_Msk) 
  {
    case ARM_USART_MODE_ASYNCHRONOUS:
      XMC_USIC_CH_SetBaudrate(uart->uart, arg, 16UL);
      break;
    
    case ARM_USART_MODE_SINGLE_WIRE:   
      XMC_USIC_CH_SetBaudrate(uart->uart, arg, 16UL);
      XMC_UART_CH_Start(uart->uart);
      
      // Configure TX pin   
      uart->pin_tx_config->mode = (XMC_GPIO_MODE_t)(XMC_GPIO_MODE_OUTPUT_OPEN_DRAIN | uart->pin_tx_alternate_function); 
#if(UC_FAMILY == XMC4)

      uart->pin_tx_config->output_strength = XMC_GPIO_OUTPUT_STRENGTH_STRONG_MEDIUM_EDGE;
#endif
      break;
    
    // Control TX
    case ARM_USART_CONTROL_TX:
      if (arg) 
      {
        XMC_UART_CH_Start(uart->uart);
        uart->info->flags |= UART_TX_ENABLED;
        if ((uart->info->mode != ARM_USART_MODE_SINGLE_WIRE )) 
        {
          uart->pin_tx_config->mode =(XMC_GPIO_MODE_t)(XMC_GPIO_MODE_OUTPUT_PUSH_PULL | uart->pin_tx_alternate_function);
        }
#if(UC_FAMILY == XMC4)

        uart->pin_tx_config->output_strength = XMC_GPIO_OUTPUT_STRENGTH_STRONG_MEDIUM_EDGE;
#endif
        // Configure TX pin 
        XMC_GPIO_Init(uart->pin_tx.port,uart->pin_tx.pin, uart->pin_tx_config); 
      } 
      else
      {
        uart->info->flags &= ~UART_TX_ENABLED;  
      }
     
      return ARM_DRIVER_OK;

    // Control RX
    case ARM_USART_CONTROL_RX:
      XMC_UART_CH_Start(uart->uart);
      if (arg) {
        if ((uart->info->mode != ARM_USART_MODE_SINGLE_WIRE )) 
        {
          // USART RX pin function selected
          uart->pin_rx_config->mode = XMC_GPIO_MODE_INPUT_PULL_UP;
          XMC_GPIO_Init(uart->pin_rx.port,uart->pin_rx.pin, uart->pin_rx_config);
        }
        uart->info->flags |= UART_RX_ENABLED;
      }
      else
      {
        uart->info->flags &= ~UART_RX_ENABLED;
      }
  
      return ARM_DRIVER_OK;

    // Abort Send
    case ARM_USART_ABORT_SEND:
      // Disable transmit holding register empty interrupt
      XMC_USIC_CH_TXFIFO_DisableEvent(uart->uart,XMC_USIC_CH_TXFIFO_EVENT_CONF_STANDARD); 
      XMC_USIC_CH_TXFIFO_Flush(uart->uart);
      // Clear Send active flag
      uart->info->status.tx_busy=0;
    return ARM_DRIVER_OK;

    // Abort receive
    case ARM_USART_ABORT_RECEIVE:
       // Disable receive data available interrupt
       XMC_USIC_CH_RXFIFO_DisableEvent(uart->uart,XMC_USIC_CH_RXFIFO_EVENT_CONF_STANDARD);
       XMC_USIC_CH_RXFIFO_DisableEvent(uart->uart,XMC_USIC_CH_RXFIFO_EVENT_CONF_ALTERNATE);
       XMC_USIC_CH_RXFIFO_Flush(uart->uart);
       // Clear RX busy status
       uart->info->status.rx_busy = 0;
      return ARM_DRIVER_OK;

    case ARM_USART_ABORT_TRANSFER:
    case ARM_USART_CONTROL_BREAK:
    case ARM_USART_MODE_SYNCHRONOUS_SLAVE:
    case ARM_USART_MODE_SYNCHRONOUS_MASTER:
    case ARM_USART_MODE_IRDA:
    case ARM_USART_MODE_SMART_CARD:   
    case ARM_USART_SET_DEFAULT_TX_VALUE:
    case ARM_USART_SET_IRDA_PULSE:
    case ARM_USART_SET_SMART_CARD_GUARD_TIME:
    case ARM_USART_SET_SMART_CARD_CLOCK:
    case ARM_USART_CONTROL_SMART_CARD_NACK:
      return ARM_USART_ERROR_MODE;

    // Unsupported command
    default: 
      return ARM_DRIVER_ERROR_UNSUPPORTED;
  }
  
  // USART Data bits
  switch (control & ARM_USART_DATA_BITS_Msk) 
  {
    case ARM_USART_DATA_BITS_5: 
      XMC_UART_CH_SetWordLength(uart->uart, 5);
      XMC_UART_CH_SetFrameLength(uart->uart, 5);
      break;
    
    case ARM_USART_DATA_BITS_6: 
      XMC_UART_CH_SetWordLength(uart->uart, 6);
      XMC_UART_CH_SetFrameLength(uart->uart, 6);
      break;
    
    case ARM_USART_DATA_BITS_7: 
      XMC_UART_CH_SetWordLength(uart->uart, 7);
      XMC_UART_CH_SetFrameLength(uart->uart, 7);
      break;
    
    case ARM_USART_DATA_BITS_8: 
      XMC_UART_CH_SetWordLength(uart->uart, 8);
      XMC_UART_CH_SetFrameLength(uart->uart, 8);
      break;
    
    case ARM_USART_DATA_BITS_9: 
      XMC_UART_CH_SetWordLength(uart->uart, 9);
      XMC_UART_CH_SetFrameLength(uart->uart, 9); 
      break;
    
    default: 
      return ARM_USART_ERROR_DATA_BITS;
  }

  // UART Parity
  switch (control & ARM_USART_PARITY_Msk) 
  { 
    case ARM_USART_PARITY_NONE:  
      uart->uart->CCR = XMC_USIC_CH_PARITY_MODE_NONE; 
      break;
    
    case ARM_USART_PARITY_EVEN:  
      uart->uart->CCR = XMC_USIC_CH_PARITY_MODE_EVEN; 
      break;
    
    case ARM_USART_PARITY_ODD:  
      uart->uart->CCR = XMC_USIC_CH_PARITY_MODE_ODD; 
      break;
  }

  // USART Stop bits
  switch (control & ARM_USART_STOP_BITS_Msk) 
  {
    case ARM_USART_STOP_BITS_1:   
      uart->uart->PCR_ASCMode &= ~((uint32_t)USIC_CH_PCR_ASCMode_STPB_Msk);   
      break;
    
    case ARM_USART_STOP_BITS_2: 
      uart->uart->PCR_ASCMode &= ~((uint32_t)USIC_CH_PCR_ASCMode_STPB_Msk); 
      uart->uart->PCR_ASCMode |= ((uint32_t)USIC_CH_PCR_ASCMode_STPB_Msk); 
      break;
    
    default: 
      return ARM_USART_ERROR_STOP_BITS;
  }

// UART Flow Control
  switch (control & ARM_USART_FLOW_CONTROL_Msk) 
  {
    default: 
      break;
  }
  
// UART Clock Polarity
  switch (control & ARM_USART_CPOL_Msk) 
  {   
    default: 
      break;
  }

// UART Clock Phase
  switch (control & ARM_USART_CPHA_Msk) 
  {   
    default: 
      break;
  }

  return ARM_DRIVER_OK;
}

#if (RTE_UART0 != 0)
static int32_t UART0_Control(uint32_t control, uint32_t arg) 
{
  return UART_Control(control, arg, uart_resources[0]);
}
#endif

#if (RTE_UART1 != 0)
static int32_t UART1_Control(uint32_t control, uint32_t arg) 
{
  return UART_Control(control, arg, uart_resources[1]);
}
#endif

#if (RTE_UART2 != 0)
static int32_t UART2_Control(uint32_t control, uint32_t arg) 
{
  return UART_Control(control, arg, uart_resources[2]);
}
#endif
                                
#if (RTE_UART3 != 0)
static int32_t UART3_Control(uint32_t control, uint32_t arg) 
{
  return UART_Control(control, arg, uart_resources[3]);
}
#endif
                                
#if (RTE_UART4 != 0)
static int32_t UART4_Control(uint32_t control, uint32_t arg) 
{
  return UART_Control(control, arg, uart_resources[4]);
}
#endif

#if (RTE_UART5 != 0)
static int32_t UART5_Control(uint32_t control, uint32_t arg) 
{
  return UART_Control(control, arg, uart_resources[5]);
}
#endif
  
/**
  \fn          void UART_IRQHandler (UART_RESOURCES  *uart)
  \brief       UART Interrupt handler.
  \param[in]   uart  Pointer to UART resources
*/
static void UART_TX_ISR(UART_RESOURCES *const uart) 
{
  uint32_t tx_num = uart->info->xfer.tx_num;
  if(tx_num > uart->info->xfer.tx_cnt) 
  {
    if(uart->tx_fifo_size_reg==NO_FIFO)
    {
      uart->info->xfer.tx_cnt++;
      XMC_UART_CH_Transmit(uart->uart, *(uart->info->xfer.tx_buf++));
    }
    else 
    {
      /* Write to FIFO till Fifo is full */
      while((XMC_USIC_CH_TXFIFO_IsFull(uart->uart) == false))
      {   
        if(uart->info->xfer.tx_cnt < tx_num) 
        { 
          uart->info->xfer.tx_cnt++;
          XMC_USIC_CH_TXFIFO_PutData(uart->uart, *(uart->info->xfer.tx_buf++));
        }
        else
        {
          break;
        }
      }
    }
  }
  
  if(tx_num == uart->info->xfer.tx_cnt) 
  {
    if(XMC_USIC_CH_TXFIFO_IsEmpty(uart->uart) == true)
    { 
      if(uart->tx_fifo_size_reg==NO_FIFO)
      {
        /* Disable standard transmit and error event interrupt */
        XMC_UART_CH_DisableEvent(uart->uart, XMC_UART_CH_EVENT_TRANSMIT_BUFFER);
      }
      else
      {
        /* Disable standard transmit and error event interrupt */
        XMC_USIC_CH_TXFIFO_DisableEvent(uart->uart, XMC_USIC_CH_TXFIFO_EVENT_CONF_STANDARD);
      }
      while((XMC_USIC_CH_GetTransmitBufferStatus(uart->uart)& XMC_USIC_CH_TBUF_STATUS_BUSY)){};

      // Clear TX busy flag
      uart->info->status.tx_busy=0; 
      if (uart->info->cb_event) uart->info->cb_event(ARM_USART_EVENT_SEND_COMPLETE);      
    }
  }
}

static void UART_RX_ISR(UART_RESOURCES *const uart) 
{
  uint32_t rx_num = uart->info->xfer.rx_num;
  if(uart->rx_fifo_size_reg == NO_FIFO)
  {
    uart->info->xfer.rx_cnt++;
    *(uart->info->xfer.rx_buf++) = (uint8_t)XMC_UART_CH_GetReceivedData(uart->uart);

    if(uart->info->xfer.rx_cnt == rx_num)
    {       
      XMC_UART_CH_DisableEvent(uart->uart, XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE);

      // Clear RX busy flag and set receive transfer complete event
      uart->info->status.rx_busy = 0;
      if (uart->info->cb_event) uart->info->cb_event(ARM_USART_EVENT_RECEIVE_COMPLETE);  
    }
  }
  else
  {
    while(XMC_USIC_CH_RXFIFO_IsEmpty(uart->uart) == false)
    { 
      if(uart->info->xfer.rx_cnt < rx_num)
      { 
        /* Read the data from FIFO buffer */
        uart->info->xfer.rx_cnt++;
        *(uart->info->xfer.rx_buf++) = (uint8_t)XMC_USIC_CH_RXFIFO_GetData(uart->uart);
      }
      else
      {
        break;
      }
    }

    uint32_t remaining = rx_num - uart->info->xfer.rx_cnt;
    if (remaining == 0)
    {
      XMC_USIC_CH_RXFIFO_DisableEvent(uart->uart,XMC_USIC_CH_RXFIFO_EVENT_CONF_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_CONF_ALTERNATE);
      // Clear RX busy flag and set receive transfer complete event
      uart->info->status.rx_busy = 0;
      if (uart->info->cb_event) uart->info->cb_event(ARM_USART_EVENT_RECEIVE_COMPLETE); 
    }
	else
    {
      if (remaining <= uart->rx_fifo_size_num)
      {
        XMC_USIC_CH_RXFIFO_SetTriggerLimit(uart->uart, remaining - 1U);
      }
    }
  }
}

#if (RTE_UART0 != 0)
void UART0_ISR(void) 
{
#if RTE_UART0_RX_FIFO_SIZE == NO_FIFO
  if (uart_resources[0]->info->status.rx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[0]->uart) & (XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE)) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[0]->uart, XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE);
    UART_RX_ISR(uart_resources[0]);
  }
#else
  if (uart_resources[0]->info->status.rx_busy && ((XMC_USIC_CH_RXFIFO_GetEvent(uart_resources[0]->uart) & (XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE)) != 0))
  {
    XMC_USIC_CH_RXFIFO_ClearEvent(uart_resources[0]->uart, XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE);
    UART_RX_ISR(uart_resources[0]);
  }
#endif

#if RTE_UART0_TX_FIFO_SIZE == NO_FIFO
  if (uart_resources[0]->info->status.tx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[0]->uart) & XMC_UART_CH_EVENT_TRANSMIT_BUFFER) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[0]->uart, XMC_UART_CH_EVENT_TRANSMIT_BUFFER);
    UART_TX_ISR(uart_resources[0]);
  }
#else
  if (uart_resources[0]->info->status.tx_busy && ((XMC_USIC_CH_TXFIFO_GetEvent(uart_resources[0]->uart) & XMC_USIC_CH_TXFIFO_EVENT_STANDARD) != 0))
  {
    XMC_USIC_CH_TXFIFO_ClearEvent(uart_resources[0]->uart, XMC_USIC_CH_TXFIFO_EVENT_STANDARD);
    UART_TX_ISR(uart_resources[0]);
  }
#endif
}
#endif

#if (RTE_UART1 != 0)
void UART1_ISR(void) 
{
#if RTE_UART1_RX_FIFO_SIZE == NO_FIFO
  if (uart_resources[1]->info->status.rx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[1]->uart) & (XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE)) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[1]->uart, XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE);
    UART_RX_ISR(uart_resources[1]);
  }
#else
  if (uart_resources[1]->info->status.rx_busy && ((XMC_USIC_CH_RXFIFO_GetEvent(uart_resources[1]->uart) & (XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE)) != 0))
  {
    XMC_USIC_CH_RXFIFO_ClearEvent(uart_resources[1]->uart, XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE);
    UART_RX_ISR(uart_resources[1]);
  }
#endif

#if RTE_UART1_TX_FIFO_SIZE == NO_FIFO
  if (uart_resources[1]->info->status.tx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[1]->uart) & XMC_UART_CH_EVENT_TRANSMIT_BUFFER) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[1]->uart, XMC_UART_CH_EVENT_TRANSMIT_BUFFER);
    UART_TX_ISR(uart_resources[1]);
  }
#else
  if (uart_resources[1]->info->status.tx_busy && ((XMC_USIC_CH_TXFIFO_GetEvent(uart_resources[1]->uart) & XMC_USIC_CH_TXFIFO_EVENT_STANDARD) != 0))
  {
    XMC_USIC_CH_TXFIFO_ClearEvent(uart_resources[1]->uart, XMC_USIC_CH_TXFIFO_EVENT_STANDARD);
    UART_TX_ISR(uart_resources[1]);
  }
#endif
}
#endif

#if (RTE_UART2 != 0)
void UART2_ISR(void) 
{
#if RTE_UART2_RX_FIFO_SIZE == NO_FIFO
  if (uart_resources[2]->info->status.rx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[2]->uart) & (XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE)) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[2]->uart, XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE);
    UART_RX_ISR(uart_resources[2]);
  }
#else
  if (uart_resources[2]->info->status.rx_busy && ((XMC_USIC_CH_RXFIFO_GetEvent(uart_resources[2]->uart) & (XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE)) != 0))
  {
    XMC_USIC_CH_RXFIFO_ClearEvent(uart_resources[2]->uart, XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE);
    UART_RX_ISR(uart_resources[2]);
  }
#endif

#if RTE_UART2_TX_FIFO_SIZE == NO_FIFO
  if (uart_resources[2]->info->status.tx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[2]->uart) & XMC_UART_CH_EVENT_TRANSMIT_BUFFER) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[2]->uart, XMC_UART_CH_EVENT_TRANSMIT_BUFFER);
    UART_TX_ISR(uart_resources[2]);
  }
#else
  if (uart_resources[2]->info->status.tx_busy && ((XMC_USIC_CH_TXFIFO_GetEvent(uart_resources[2]->uart) & XMC_USIC_CH_TXFIFO_EVENT_STANDARD) != 0))
  {
    XMC_USIC_CH_TXFIFO_ClearEvent(uart_resources[2]->uart, XMC_USIC_CH_TXFIFO_EVENT_STANDARD);
    UART_TX_ISR(uart_resources[2]);
  }
#endif
}
#endif

#if (RTE_UART3 != 0)
void UART3_ISR(void) 
{
#if RTE_UART3_RX_FIFO_SIZE == NO_FIFO
  if (uart_resources[3]->info->status.rx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[3]->uart) & (XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE)) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[3]->uart, XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE);
    UART_RX_ISR(uart_resources[3]);
  }
#else
  if (uart_resources[3]->info->status.rx_busy && ((XMC_USIC_CH_RXFIFO_GetEvent(uart_resources[3]->uart) & (XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE)) != 0))
  {
    XMC_USIC_CH_RXFIFO_ClearEvent(uart_resources[3]->uart, XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE);
    UART_RX_ISR(uart_resources[3]);
  }
#endif

#if RTE_UART3_TX_FIFO_SIZE == NO_FIFO
  if (uart_resources[3]->info->status.tx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[3]->uart) & XMC_UART_CH_EVENT_TRANSMIT_BUFFER) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[3]->uart, XMC_UART_CH_EVENT_TRANSMIT_BUFFER);
    UART_TX_ISR(uart_resources[3]);
  }
#else
  if (uart_resources[3]->info->status.tx_busy && ((XMC_USIC_CH_TXFIFO_GetEvent(uart_resources[3]->uart) & XMC_USIC_CH_TXFIFO_EVENT_STANDARD) != 0))
  {
    XMC_USIC_CH_TXFIFO_ClearEvent(uart_resources[3]->uart, XMC_USIC_CH_TXFIFO_EVENT_STANDARD);
    UART_TX_ISR(uart_resources[3]);
  }
#endif
}
#endif

#if (RTE_UART4 != 0)
void UART4_ISR(void) 
{
#if RTE_UART4_RX_FIFO_SIZE == NO_FIFO
  if (uart_resources[4]->info->status.rx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[4]->uart) & (XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE)) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[4]->uart, XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE);
    UART_RX_ISR(uart_resources[4]);
  }
#else
  if (uart_resources[4]->info->status.rx_busy && ((XMC_USIC_CH_RXFIFO_GetEvent(uart_resources[4]->uart) & (XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE)) != 0))
  {
    XMC_USIC_CH_RXFIFO_ClearEvent(uart_resources[4]->uart, XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE);
    UART_RX_ISR(uart_resources[4]);
  }
#endif

#if RTE_UART4_TX_FIFO_SIZE == NO_FIFO
  if (uart_resources[4]->info->status.tx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[4]->uart) & XMC_UART_CH_EVENT_TRANSMIT_BUFFER) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[4]->uart, XMC_UART_CH_EVENT_TRANSMIT_BUFFER);
    UART_TX_ISR(uart_resources[4]);
  }
#else
  if (uart_resources[4]->info->status.tx_busy && ((XMC_USIC_CH_TXFIFO_GetEvent(uart_resources[4]->uart) & XMC_USIC_CH_TXFIFO_EVENT_STANDARD) != 0))
  {
    XMC_USIC_CH_TXFIFO_ClearEvent(uart_resources[4]->uart, XMC_USIC_CH_TXFIFO_EVENT_STANDARD);
    UART_TX_ISR(uart_resources[4]);
  }
#endif
}
#endif

#if (RTE_UART5 != 0)
void UART5_ISR(void) 
{
#if RTE_UART5_RX_FIFO_SIZE == NO_FIFO
  if (uart_resources[5]->info->status.rx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[5]->uart) & (XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE)) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[5]->uart, XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE);
    UART_RX_ISR(uart_resources[5]);
  }
#else
  if (uart_resources[5]->info->status.rx_busy && ((XMC_USIC_CH_RXFIFO_GetEvent(uart_resources[5]->uart) & (XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE)) != 0))
  {
    XMC_USIC_CH_RXFIFO_ClearEvent(uart_resources[5]->uart, XMC_USIC_CH_RXFIFO_EVENT_STANDARD | XMC_USIC_CH_RXFIFO_EVENT_ALTERNATE);
    UART_RX_ISR(uart_resources[5]);
  }
#endif

#if RTE_UART5_TX_FIFO_SIZE == NO_FIFO
  if (uart_resources[5]->info->status.tx_busy && ((XMC_UART_CH_GetStatusFlag(uart_resources[5]->uart) & XMC_UART_CH_EVENT_TRANSMIT_BUFFER) != 0))
  {
    XMC_UART_CH_ClearStatusFlag(uart_resources[5]->uart, XMC_UART_CH_EVENT_TRANSMIT_BUFFER);
    UART_TX_ISR(uart_resources[5]);
  }
#else
  if (uart_resources[5]->info->status.tx_busy && ((XMC_USIC_CH_TXFIFO_GetEvent(uart_resources[5]->uart) & XMC_USIC_CH_TXFIFO_EVENT_STANDARD) != 0))
  {
    XMC_USIC_CH_TXFIFO_ClearEvent(uart_resources[5]->uart, XMC_USIC_CH_TXFIFO_EVENT_STANDARD);
    UART_TX_ISR(uart_resources[5]);
  }
#endif
}
#endif

#if (RTE_UART0 != 0)
ARM_DRIVER_USART Driver_USART0 = 
{
  USARTX_GetVersion,
  UART0_GetCapabilities,
  UART0_Initialize,
  UART0_Uninitialize,
  UART0_PowerControl,
  UART0_Send, 
  UART0_Receive,
  UART0_Transfer,
  UART0_GetTxCount,
  UART0_GetRxCount,
  UART0_Control,
  UART0_GetStatus,
  UART0_SetModemControl,
  UART0_GetModemStatus
};
#endif

#if (RTE_UART1 != 0)
ARM_DRIVER_USART Driver_USART1 = 
{
  USARTX_GetVersion,
  UART1_GetCapabilities,
  UART1_Initialize,
  UART1_Uninitialize,
  UART1_PowerControl,
  UART1_Send, 
  UART1_Receive,
  UART1_Transfer,
  UART1_GetTxCount,
  UART1_GetRxCount,
  UART1_Control,
  UART1_GetStatus,
  UART1_SetModemControl,
  UART1_GetModemStatus
};
#endif

#if (RTE_UART2 != 0)
ARM_DRIVER_USART Driver_USART2 = 
{
  USARTX_GetVersion,
  UART2_GetCapabilities,
  UART2_Initialize,
  UART2_Uninitialize,
  UART2_PowerControl,
  UART2_Send, 
  UART2_Receive,
  UART2_Transfer,
  UART2_GetTxCount,
  UART2_GetRxCount,
  UART2_Control,
  UART2_GetStatus,
  UART2_SetModemControl,
  UART2_GetModemStatus
};
#endif

#if (RTE_UART3 != 0)
ARM_DRIVER_USART Driver_USART3 = 
{
  USARTX_GetVersion,
  UART3_GetCapabilities,
  UART3_Initialize,
  UART3_Uninitialize,
  UART3_PowerControl,
  UART3_Send, 
  UART3_Receive,
  UART3_Transfer,
  UART3_GetTxCount,
  UART3_GetRxCount,
  UART3_Control,
  UART3_GetStatus,
  UART3_SetModemControl,
  UART3_GetModemStatus
};
#endif

#if (RTE_UART4 != 0)
ARM_DRIVER_USART Driver_USART4 = 
{
  USARTX_GetVersion,
  UART4_GetCapabilities,
  UART4_Initialize,
  UART4_Uninitialize,
  UART4_PowerControl,
  UART4_Send, 
  UART4_Receive,
  UART4_Transfer,
  UART4_GetTxCount,
  UART4_GetRxCount,
  UART4_Control,
  UART4_GetStatus,
  UART4_SetModemControl,
  UART4_GetModemStatus
};
#endif

#if (RTE_UART5 != 0)
ARM_DRIVER_USART Driver_USART5 = 
{
  USARTX_GetVersion,
  UART5_GetCapabilities,
  UART5_Initialize,
  UART5_Uninitialize,
  UART5_PowerControl,
  UART5_Send, 
  UART5_Receive,
  UART5_Transfer,
  UART5_GetTxCount,
  UART5_GetRxCount,
  UART5_Control,
  UART5_GetStatus,
  UART5_SetModemControl,
  UART5_GetModemStatus
};
#endif

#endif
